from juriscraper.opinions.united_states.state import nytrial


class Site(nytrial.Site):
    court_regex = r"Sup[rt]?\.? ?[Cc]o?u?r?t?|[sS]ur?pu?rem?e? C(our)?t|Sur?pe?r?me?|Suoreme|Sup County|Integrated Domestic Violence|Soho Fashions LTD"



"""
Scraper for the Supreme Court of Tennessee
CourtID: tenn
Court Short Name: Tenn.
"""


from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.tncourts.gov/courts/supreme-court/opinions"
        self.court_id = self.__module__
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath("//tr"):
            date = (
                row.xpath(
                    ".//td[contains(@class, 'views-field-field-opinions-date-filed')]"
                )[0]
                .text_content()
                .strip()
            )
            lower_court = (
                row.xpath(
                    ".//td[contains(@class, 'views-field-field-opinions-county')]"
                )[0]
                .text_content()
                .strip()
            )
            section = row.xpath(
                ".//td[contains(@class, 'views-field-field-opinions-case-number')]"
            )[0]
            url = section.xpath(".//a")[0].get("href")
            name = section.xpath(".//a")[0].text_content()
            rows = [
                row.strip()
                for row in section.text_content().strip().split("\n", 4)
            ]
            self.cases.append(
                {
                    "date": date,
                    "lower_court": lower_court,
                    "url": url,
                    "name": name,
                    "docket": rows[1],
                    "judges": rows[2].split(": ")[1],
                    "summary": rows[-1],
                }
            )



"""Scraper for the California Attorney General
CourtID: calag
Court Short Name: California Attorney General
"""

import datetime

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = datetime.date.today().year
        self.url_base = "https://oag.ca.gov/opinions/yearly-index?conclusion-year[value][year]="
        self.url = self.url_base + str(self.year)
        self.back_scrape_iterable = list(range(1985, self.year + 1))
        self.rows_path = '//tbody/tr[contains(./td[1]//a/@href, ".pdf")]'
        self.cell_path = f"{self.rows_path}/td[%d]"

    def _get_case_names(self):
        """No case names available"""
        return ["Untitled California Attorney General Opinion"] * len(
            self.html.xpath(self.rows_path)
        )

    def _get_download_urls(self):
        path = f"{self.cell_path % 1}//a/@href"
        return [href for href in self.html.xpath(path)]

    def _get_case_dates(self):
        dates = []
        for cell in self.html.xpath(self.cell_path % 4):
            date_raw = cell.text_content().replace(r"\n", "").strip()
            dates.append(convert_date_string(date_raw))
        return dates

    def _get_docket_numbers(self):
        return [
            cell.text_content().strip()
            for cell in self.html.xpath(self.cell_path % 1)
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_summaries(self):
        """Combine the Questions and Conclusions Columns"""
        summaries = []
        for row in self.html.xpath(self.rows_path):
            questions = row.xpath("./td[2]")[0].text_content()
            conclusions = row.xpath("./td[3]")[0].text_content()
            summaries.append(
                f"QUESTIONS: {questions} CONCLUSIONS: {conclusions}"
            )
        return summaries

    def _download_backwards(self, year):
        self.url = self.url_base + str(year)
        self.html = self._download()



# Scraper for Florida 6th District Court of Appeal
# CourtID: flaapp6
# Court Short Name: flaapp6

from . import fladistctapp


class Site(fladistctapp.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.number = "sixth"
        self.base = "https://6dca.flcourts.gov"
        self.update_url()



import re
from datetime import date, datetime, timedelta
from urllib.parse import urlencode

from dateutil.rrule import DAILY, rrule

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.base_url = "http://media.ca1.uscourts.gov/cgi-bin/opinions.pl"
        self.court_id = self.__module__
        today = date.today()
        params = urlencode(
            {
                "FROMDATE": (today - timedelta(7)).strftime("%m/%d/%Y"),
                "TODATE": today.strftime("%m/%d/%Y"),
                "puid": "",
            }
        )
        self.url = f"{self.base_url}/?{params}"
        # self.url = "http://media.ca1.uscourts.gov/cgi-bin/opinions.pl/?TODATE=06%2F24%2F1993&puid=&FROMDATE=05%2F25%2F1993"
        self.interval = 30
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=self.interval,
                dtstart=date(1992, 1, 1),
                until=date(2016, 1, 1),
            )
        ]

    def _get_case_names(self):
        return [
            e.strip()
            for e in self.html.xpath(
                "//tr[position() > 1]/td[4]/text()[contains(., 'v.')]"
            )
        ]

    def _get_download_urls(self):
        return [
            e for e in self.html.xpath("//tr[position() > 1]/td[2]//@href")
        ]

    def _get_case_dates(self):
        dates = []
        for s in self.html.xpath("//tr[position() > 1]/td[1]//text()"):
            s = s.replace(r"\t", "").replace(r"\n", "").strip()
            if s == "1996/05/32":
                s = "1996/05/30"  # My life is thus lain to waste.
            dates.append(datetime.strptime(s.strip(), "%Y/%m/%d").date())
        return dates

    def _get_docket_numbers(self):
        regex = re.compile(r"(\d{2}-.*?\W)(.*)$")
        docket_numbers = []
        for s in self.html.xpath("//tr[position() > 1]/td[2]/a/text()"):
            s = s.replace("O1-", "01-")  # I grow older, the input grows worse.
            docket_numbers.append(
                regex.search(s).group(1).strip().replace(".", "")
            )
        return docket_numbers

    def _get_precedential_statuses(self):
        statuses = []
        for text in self.html.xpath("//tr[position() > 1]/td[2]//@href"):
            if "U" in text:
                statuses.append("Unpublished")
            elif "P" in text:
                statuses.append("Published")
            elif "E" in text:
                statuses.append("Errata")
            else:
                statuses.append("Unknown")
        return statuses

    def _get_lower_courts(self):
        lower_courts = []
        for e in self.html.xpath("//tr[position() > 1]/td[4]/font"):
            try:
                lower_courts.append(e.xpath("./text()")[0].strip())
            except IndexError:
                lower_courts.append("")
        return lower_courts

    def _download_backwards(self, d):
        params = urlencode(
            {
                "FROMDATE": d.strftime("%m/%d/%Y"),
                "TODATE": (d + timedelta(self.interval)).strftime("%m/%d/%Y"),
                "puid": "",
            }
        )
        self.url = f"{self.base_url}/?{params}"

        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200

    def _post_parse(self):
        """This will remove the cases without a case name"""
        to_be_removed = [
            index
            for index, case_name in enumerate(self.case_names)
            if not case_name.replace("v.", "").strip()
        ]

        for attr in self._all_attrs:
            item = getattr(self, attr)
            if item is not None:
                new_item = self.remove_elements(item, to_be_removed)
                self.__setattr__(attr, new_item)

    @staticmethod
    def remove_elements(list_, indexes_to_be_removed):
        return [
            i for j, i in enumerate(list_) if j not in indexes_to_be_removed
        ]



"""
Author: William Palin
Date created: 2023-01-21
Scraper for the Decisiones del Tribunal Apelaciones
CourtID: prapp
Court Short Name: Puerto Rico Court of Apelaciones
"""
from dateparser import parse

from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = f"https://poderjudicial.pr/tribunal-apelaciones/decisiones-finales-del-tribunal-de-apelaciones/"
        self.status = "Published"

    def _download(self, request_dict={}):
        """Download websites

        :param request_dict: Empty dict
        :return: HTML object
        """
        if self.test_mode_enabled():
            return super()._download()
        if not self.html:
            self.html = super()._download()
        self.url = self.html.xpath(
            ".//ul[1]/li/a[contains(@href, 'decisiones-finales-del-tribunal-de-apelaciones/decisiones-del-tribunal-de-apelaciones')]/../.."
        )[0].xpath(".//a/@href")[-1]
        return super()._download()

    def _process_html(self):
        for row in self.html.xpath(".//table/tbody/tr"):
            cells = row.xpath(".//td")
            maybe_link = cells[0].xpath(".//a/@href")
            if not maybe_link:
                continue
            else:
                url = maybe_link[0]
            date_obj = parse(cells[2].text_content(), languages=["es"])

            self.cases.append(
                {
                    "name": titlecase(cells[1].text_content()),
                    "url": url,
                    "docket": cells[0].text_content(),
                    "date": str(date_obj.date()),
                }
            )



"""
Scraper for Massachusetts Appeals Court Rule 23.0 (formerly Rule 1:28)
CourtID: massappct
Court Short Name: Mass App Ct
Author: William Palin
Court Contact: SJCReporter@sjc.state.ma.us (617) 557-1030
Reviewer:
Date: 2020-02-27
"""

from datetime import date, timedelta
from urllib.parse import urlencode

from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.case_date = date.today()
        self.backwards_days = 14
        self.base = "https://128archive.com"
        self.url = self.make_url()
        self.court_id = self.__module__

    def make_url(self) -> str:
        """Build the urls to query

        :return: The url parameters (encoded)
        """
        start_date = self.case_date - timedelta(days=self.backwards_days)
        first_date = start_date.strftime("%m/%d/%Y")
        end_date = self.case_date.strftime("%m/%d/%Y")
        self.parameters = (
            ("Action", "search"),
            ("DocketNumber", ""),
            ("PartyName", ""),
            ("ReleaseDateFrom", first_date),
            ("ReleaseDateTo", end_date),
            ("Keywords", ""),
            (
                "SortColumnName",
                [
                    "Release Date Descending Order",
                    "Release Date Descending Order",
                ],
            ),
            ("SortOrder", ""),
            ("CurrentPageNo", "1"),
            ("Pages", "1"),
            ("PageSize", "100"),
        )
        return f"{self.base}/?{urlencode(self.parameters)}"

    def _process_html(self) -> None:
        """Process the data

        :return: None
        """
        for row in self.html.xpath("//div[@data-rowtype]"):
            docket, name, date = row.xpath(
                ".//div[@class='col-md-7 font-bold']/text()"
            )
            path = row.xpath(".//div/div/a/@href")[0]
            self.cases.append(
                {
                    "date": date.strip(),
                    "docket": docket.strip(),
                    "name": titlecase(name.strip()),
                    "url": path,
                    "status": "Unpublished",
                }
            )



"""Scraper for Public Interest Cases for the CADC
CourtID: cadc
Court Short Name: Court of Appeals of the District of Columbia
Author: flooie
History:
  2021-12-18: Created by flooie
  2023-01-12: Fixed requests.exceptions.InvalidURL error by grossir
"""
from urllib.parse import urljoin

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.cadc.uscourts.gov/internet/orders.nsf"
        self.base = "https://www.cadc.uscourts.gov"
        self.court_id = self.__module__

    def _process_html(self) -> None:
        """Iterate over the public interest cases.

        :return: None
        """
        for row in self.html.xpath(".//div[@class='row-entry']"):
            url = row.xpath(".//a/@href")[0]
            docket = row.xpath(".//a/span/text()")[0]
            name = row.xpath(".//div[@class='column-two']/div[1]/text()")[
                0
            ].strip()
            date = row.xpath(".//date/text()")[0]
            self.cases.append(
                {
                    "date": date,
                    "url": urljoin("https:", url),
                    "docket": docket,
                    "name": name,
                    "status": "Published",
                }
            )



# Author: Michael Lissner
# Date created: 2013-06-21

from juriscraper.opinions.united_states.state import wva


class Site(wva.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.courtswv.gov/intermediate-court/opinions.html"



# Scraper for New York Appellate Divisions 1st Dept.
# CourtID: nyappdiv_1st
# Court Short Name: NY
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-04

from datetime import date

from dateutil.rrule import MONTHLY, rrule

from juriscraper.AbstractSite import logger
from juriscraper.lib.html_utils import get_html5_parsed_text
from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        date_keys = rrule(
            MONTHLY, dtstart=date(2003, 11, 1), until=date(2015, 8, 30)
        )
        self.back_scrape_iterable = [i.date() for i in date_keys]
        self.row_base_path = '//tr[contains(./td[1]/a/@href, "3d")]'
        self.division = 1
        self.url = self.build_url()

    def _get_case_names(self):
        path = f"{self.row_base_path}/td[1]"
        return [cell.text_content() for cell in self.html.xpath(path)]

    def build_url(self, target_date=False):
        base = (
            "http://www.courts.state.ny.us/reporter/slipidx/aidxtable_%s"
            % self.division
        )
        if target_date:
            return "{}_{}_{}.shtml".format(
                base,
                target_date.year,
                target_date.strftime("%B"),
            )
        else:
            return f"{base}.shtml"

    def _get_download_urls(self):
        path = f"{self.row_base_path}/td[1]//a/@href"
        return self.html.xpath(path)

    def _get_case_dates(self):
        case_dates = []
        for element in self.html.xpath("//caption | //center"):
            date_string = (
                element.text_content().strip().replace("Cases Decided ", "")
            )
            path_prefix = (
                "./parent::"
                if element.tag == "caption"
                else "./following-sibling::"
            )
            path = f"{path_prefix}table[1]{self.row_base_path}"
            cases = element.xpath(path)
            case_dates.extend([convert_date_string(date_string)] * len(cases))
        return case_dates

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        path = f"{self.row_base_path}/td[3]"
        return list(
            map(
                self._add_str_to_list_where_empty_element,
                self.html.xpath(path),
            )
        )

    def _get_judges(self):
        path = f"{self.row_base_path}/td[2]"
        return list(
            map(
                self._add_str_to_list_where_empty_element,
                self.html.xpath(path),
            )
        )

    def _get_citations(self):
        path = f"{self.row_base_path}/td[4]"
        return [cell.text_content().strip() for cell in self.html.xpath(path)]

    @staticmethod
    def _add_str_to_list_where_empty_element(element):
        string_list = element.xpath("./text()")
        return string_list[0] if string_list else ""

    def _download_backwards(self, target_date):
        self.crawl_date = target_date
        logger.info(f"Running backscraper with date: {target_date}")
        self.url = self.build_url(target_date=target_date)
        self.html = self._download()

    def _make_html_tree(self, text):
        return get_html5_parsed_text(text)



"""Scraper for the Vermont Environmental
CourtID: vt
Court Short Name: VT
Court Contact: submit form here https://www.vermontjudiciary.org/website-feedback-form
"""

from . import vt


class Site(vt.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.division = 4
        self.url = f"https://www.vermontjudiciary.org/opinions-decisions?f%5B0%5D=court_division_opinions_library_%3A{self.division}"  # supreme



from juriscraper.opinions.united_states.state import nj


class Site(nj.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "https://www.njcourts.gov/attorneys/opinions/published-appellate"
        )
        self.status = "Published"



"""Scraper for Army Court of Criminal Appeals
CourtID: acca
Reviewer: None
History:
  2015-01-08: Created by mlr
  2016-03-17: Website appears to be dead. Scraper disabled in __init__.py.
  2019-09-11: Rewritten by arderyp to user Linear pattern
"""

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.jagcnet.army.mil/ACCALibrary/cases/opinions/OC"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath(
            '//*[@id="Opinions_ResizeContainer"]/table/tbody/tr'
        ):
            col1, col2, col3 = row.xpath(".//td")
            url = col1.xpath(".//a/@href")
            if not url:
                continue
            name = col1.text_content()
            docket = col2.text_content()
            date = col3.text_content()
            self.cases.append(
                {
                    "date": date.strip(),
                    "docket": docket.strip(),
                    "name": name.strip(),
                    "url": url[0],
                }
            )



"""
CourtID:	ca6
Court Contact:	WebSupport@ca6.uscourts.gov
"""

from juriscraper.lib.string_utils import clean_if_py3, convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.opn.ca6.uscourts.gov/opinions/opinions.php"

    def _get_case_names(self):
        return [
            string.split(" - ")[0]
            for string in self.get_nth_table_cell_data(4)
        ]

    def _get_download_urls(self):
        return self.get_nth_table_cell_data(1, href=True)

    def _get_case_dates(self):
        return [
            convert_date_string(date)
            for date in self.get_nth_table_cell_data(3)
        ]

    def _get_docket_numbers(self):
        return self.get_nth_table_cell_data(2)

    def _get_precedential_statuses(self):
        statuses = []
        for file_name in self.get_nth_table_cell_data(1, link_text=True):
            if "n" in file_name.lower():
                statuses.append("Unpublished")
            elif "p" in file_name.lower():
                statuses.append("Published")
            else:
                statuses.append("Unknown")
        return statuses

    def get_nth_table_cell_data(self, n, href=False, link_text=False):
        path = "//table/tr/td[%d]" % n
        if href:
            path += "/a/@href"
        elif link_text:
            path += "/a/text()"
        else:
            path += "/text()"

        results = []
        for data in self.html.xpath(path):
            data = clean_if_py3(data).strip()
            if data:
                results.append(data)
        return results



"""Scraper for Armed Services Board of Contract Appeals
CourtID: asbca
Court Short Name: ASBCA
Author: Jon Andersen
Reviewer: mlr
History:
    2014-09-11: Created by Jon Andersen
    2016-03-17: Website and phone are dead. Scraper disabled in __init__.py.
"""

from datetime import datetime

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "http://www.asbca.mil/Decisions/decisions%d.html"
            % datetime.today().year
        )
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath(".//tr")[1:]:
            col1, col2, col3, col4 = row.xpath(".//td")
            date = col1.text_content().strip()
            if not col2.text_content().strip():
                continue
            name = col3.text_content().strip()
            docket = col2.text_content().strip()
            url = col3.xpath(".//a/@href")[0]
            self.cases.append(
                {"date": date, "name": name, "url": url, "docket": docket}
            )



"""Scraper for U.S. Merit Systems Protection Board
CourtID: mspb
Court Short Name: MSPB
Author: Jon Andersen
Reviewer: mlr
Date created: 1 Sep 2014
Type: Non-precedential
"""

from juriscraper.opinions.united_states.administrative_agency import mspb_p


class Site(mspb_p.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Unpublished"
        self.url = "https://www.mspb.gov/decisions/nonprecedential/NonPrecedentialDecisions_Manifest_Table.json"
        self.base = "https://www.mspb.gov/decisions/nonprecedential/"



import json
import pprint
import re
import sys
from collections import OrderedDict

from ..lib.html_utils import strip_bad_html_tags_insecure
from ..lib.log_tools import make_default_logger
from ..lib.string_utils import convert_date_string
from .appellate_docket import AppellateDocketReport

logger = make_default_logger()


class ACMSDocketReport(AppellateDocketReport):
    """Parse ACMS docket JSON."""

    def __init__(self, court_id, pacer_session=None):
        super().__init__(court_id, pacer_session)
        self._acms_json = None

    def _parse_text(self, text):
        """Store the ACMS JSON

        This does not, in fact, actually *parse* the data, it
        stores it for subsequent parsing, which happens in
        data(), parties(), and docket_entries().

        :param text: A unicode object
        :return: None
        """
        self._acms_json = json.loads(text)

    @property
    def metadata(self):
        """Metadata is the basis returned by BaseDocketReport's data property.

        That function then supplements it with parties and docket entries.
        """

        caseDetails = self._acms_json["caseDetails"]

        data = {
            "court_id": self.court_id,
            "pacer_case_id": caseDetails["caseId"],
            "docket_number": caseDetails["caseNumber"],
            "case_name": caseDetails["name"],
            "date_filed": convert_date_string(caseDetails["caseOpened"]),
            "appeal_from": caseDetails["court"]["name"],
            "fee_status": caseDetails["feeStatus"],
        }

        # originating_court_infomation, see AppellateDocketReport
        ogc_info = {}

        # Surprisingly, we lack the lower court docket number. (!)

        # At the moment, we lack a good way to turn the contents of the `court`
        # JSON Object into a `court_id`. Nor the `districtCourtName`
        # so for now, return these things:
        #
        # court: { name: "E.D.N.Y. (BROOKLYN)",
        #          identifier: "E.D.N.Y. BROOKLYN)" }
        # court:  { name: "S.D.N.Y. (WHITE PLAINS)",
        #           identifier: "S.D.N.Y. (WHITE PLAINS)" }
        # court: { name: "S.D.N.Y . (NEW YORK CITY)",
        #          identifier: "S.D.N.Y. (NEW YORK CITY)" }
        # districtCourtName: "San Francisco Northern California"
        # districtCourtName: "Los Angeles Central California"
        if caseDetails["court"]:
            court = caseDetails["court"]
            if "name" in court:
                ogc_info["name"] = court["name"]
            if "identifier" in court:
                ogc_info["identifier"] = court["identifier"]
        if caseDetails["districtCourtName"]:
            ogc_info["name"] = caseDetails["districtCourtName"]

        if caseDetails["aNumber"]:
            ogc_info["RESTRICTED_ALIEN_NUMBER"] = caseDetails["aNumber"]

        if ogc_info:
            data["originating_court_information"] = ogc_info

        # Comma-seperated list of non-null case types, subtypes, &c.
        case_type_keys = ["caseType", "caseSubType", "caseSubSubType"]
        case_types = [caseDetails[_] for _ in case_type_keys if caseDetails[_]]
        data["case_type_information"] = ", ".join(case_types)

        # This is unwise and unreasonably general, cf. lib/utils.py
        # # data = clean_court_object(data)

        return data

    @property
    def parties(self):
        """Parse and return a dict of party/atty info.

        This parses the HTML embeded in the docket entry JSON provided
        by ACMS.
        """

        # Simple example HTML, from ca2 23-7222
        _ = (
            """
<table width='100%'>
<tr>
  <td width='30%' style='padding-bottom: 20px;'>
    UNITED STATES OF AMERICA<br>
    &nbsp;&nbsp;&nbsp;&nbsp;AppelleeUSA, """
            + """
  </td><td style='padding-bottom: 20px;'>
    <div style='margin-bottom: 10px'>Won S. Shin, Assistant U.S. Attorney
    <br>Email: won.shin@usdoj.gov
    <br>[US Attorney]
    <br>United States Attorney's Office for the Southern District of New York
    <br>One Saint Andrew's Plaza
    <br>New York, NY 10007
    </div>
  </td>
</tr><tr>
  <td width='30%' style='padding-bottom: 20px;'> MUSTAPHA RAJI<br>
    &nbsp;&nbsp;&nbsp;&nbsp;AKA Sealed Defendant 1, <br>
    &nbsp;&nbsp;&nbsp;&nbsp;Appellant, """
            + """
  </td><td style='padding-bottom: 20px;'>
    <div style='margin-bottom: 10px'>Jeremy Schneider, -
    <br>Direct: 212-571-5500
    <br>Email: jschneider@rssslaaw.com
    <br>[CJA Appointment]
    <br>Rothman, Schneider, Soloway & Stern, LLP
    <br>100 Lafayette Street
    <br>Suite 501
    <br>New York, NY 10013
    </div>
  </td>
</tr>
</table>
"""
        )

        # Slightly more complicated HTML, also ca2: ca2-23-7246.json
        _ = (
            """
<table width='100%'>
<tr>
  <td width='30%' style='padding-bottom: 20px;'>
     ASCENT PHARMACEUTICALS, INC.<br>
    &nbsp;&nbsp;&nbsp;&nbsp;Petitioner, """
            + """
  </td>
  <td style='padding-bottom: 20px;'>
    <div style='margin-bottom: 10px'>James A. Walden, -<br>
      Direct: 212-335-2031<br>
      Email: jwalden@wmhlaw.com<br>
      [Retained]<br>
      Walden Macht & Haran LLP<br>
      250 Vesey Street<br>
      27th Floor<br>
      New York, NY 10281
    </div>
  </td>
</tr>
<tr>
  <td width='30%' style='padding-bottom: 20px;'>
     UNITED STATES DRUG ENFORCEMENT ADMINISTRATION<br>
    &nbsp;&nbsp;&nbsp;&nbsp;Respondent, """
            + """
  </td>
  <td style='padding-bottom: 20px;'>
    <div style='margin-bottom: 10px'>
      Urja Mittal, -<br>
      Direct: 202-353-4895<br>
      Email: urja.mittal@usdoj.gov<br>
      [US Attorney]<br>
      United States Department of Justice<br>
      Civil Division, Appellate Staff<br>
      950 Pennsylvania Avenue, NW<br>
      Washington, DC 20530
    </div>
    <div style='margin-bottom: 10px'>
      Mark B. Stern, -<br>
      Email: mark.stern@usdoj.gov<br>
      [US Attorney]<br>
      United States Department of Justice<br>
      Civil Division, Office of Immigration Litigation<br>
      950 Pennsylvania Avenue, NW<br>
      Washington, DC 20530
    </div>
  <div style='margin-bottom: 10px'>
     United States Drug Enforcement Administration<br>
  </div>
  </td>
</tr>
<tr>
  <td width='30%' style='padding-bottom: 20px;'>
     UNITED STATES DEPARTMENT OF JUSTICE<br>
    <strong>Terminated:</strong>
    10/06/2023<br>
    &nbsp;&nbsp;&nbsp;&nbsp;Respondent, """
            + """
  </td>
  <td style='padding-bottom: 20px;'>
  <div style='margin-bottom: 10px'>
    Urja Mittal, -<br>
    <strong>Terminated:</strong> 10/06/2023<br>
    Direct: 202-353-4895<br>
    Email: urja.mittal@usdoj.gov<br>
    [US Attorney]<br>
    United States Department of Justice<br>
    Civil Division, Appellate Staff<br>
    950 Pennsylvania Avenue, NW<br>
    Washington, DC 20530
  </div>
  <div style='margin-bottom: 10px'>
    Mark B. Stern, -<br>
    <strong>Terminated:</strong> 10/06/2023<br>
    Email: mark.stern@usdoj.gov<br>
    [US Attorney]<br>
    United States Department of Justice<br>
    Civil Division, Office of Immigration Litigation<br>
    950 Pennsylvania Avenue, NW<br>
    Washington, DC 20530
  </div>
  <div style='margin-bottom: 10px'>
    Benjamin H. Torrance, Assistant U.S. Attorney<br>
    <strong>Terminated:</strong> 10/05/2023<br>
    Email: benjamin.torrance@usdoj.gov<br>
    [US Attorney]<br>
    United States Attorney's Office for the Southern District of New York<br>
    86 Chambers Street<br>
    New York, NY 10007
  </div>
  <div style='margin-bottom: 10px'>
    Lena D Watkins, Trial Attorney<br>
    <strong>Terminated:</strong> 10/05/2023<br>
    Direct: 202-514-8713<br>
    Email: Lena.Watkins@usdoj.gov<br>
    [US Attorney]<br>
    United States Department of Justice<br>
    CRM/Narc. & Dang. Drug. Sec.<br>
    145 N Street, NE<br>
    Second Floor, East W<br>
    Washington, DC 20530
  </div>
  </td>
</tr>
</table>"""
        )

        # HTML is a <table> with <tr> for each party.
        # Within that, the first <td> is the party name.
        # The second <td> is the atty list.
        # Within the atty list, there is one <div> per attorney.

        parties_html = self._acms_json["caseDetails"]["partyAttorneyList"]
        tree = strip_bad_html_tags_insecure(parties_html)
        party_rows = tree.xpath(".//tr")
        parties = []
        for row in party_rows:
            party = OrderedDict()

            (party_left, attorneys_block) = row.xpath(".//td")

            party.update(self._parse_party_left(party_left))

            attorneys = []
            for attorney in attorneys_block:
                _ = self._parse_attorney(attorney)
                # Sometimes there are empty padding lines, e.g.:
                #   <div><p>&nbsp;</p></div>
                # Ignore them.
                attorneys.append(_) if _ else None

            party["attorneys"] = attorneys

            parties.append(party)

        # Assert there is no "SEE ABOVE" in ACMS text, so no need
        # to call `self.normalalize_see_above_attorneys()`
        for party in parties:
            for attorney in party.get("attorneys", []):
                if re.search(r"see\s+above", attorney.get("attorneys", "")):
                    assert "Unexpected SEE ABOVE found in ACMS attorney list"

        return parties

    @property
    def docket_entries(self):
        """Return a dictionary of docket entries

        Format this for courtlistener's `DocketEntry` class
        (as mediated by `add_docket_entry()` in `cl/recap/mergers.py`).
        Unfortunately that class conflates filing and entry dates
        right now.  We do not have filing dates, only entry dates (and
        other dates that do not exist in our model).  Return the entry
        datetime as filing date, as well as continuing to do so for a
        future entry date.

        Unlike other docket parsers, we return this as HTML, as per
        @mlissner it is fine to do so, and there is semantic content
        to the HTML markup we would otherwise lose.

        """

        docket_entries = []
        for row in self._acms_json["docketInfo"]["docketEntries"]:
            _ = """Here's what we have to work with:
{
  "endDate": "2023-10-05",
  "endDateFormatted": "10/05/2023",
  "entryNumber": 19,
  "docketEntryText": "<p>NEW PARTY, Respondent United States Drug Enforcement Administration for United States Department of Justice, SUBSTITUTED. [Entered: 10/05/2023 03:56 PM] [Edited: 10/06/2023 10:51 AM]</p>",
  "docketEntryId": "a6a6d3d6-b863-ee11-be6e-001dd804ed2e",
  "createdOn": "2023-10-05T19:53:24Z",
  "documentCount": 0,
  "pageCount": 0,
  "fileSize": 0,
  "restrictedPartyFilingDocketEntry": false,
  "restrictedDocsAvailable": false,
  "selected": false
},"""

            de = {}
            de["document_number"] = row["entryNumber"]

            # Return HTML, currently unused.
            de["description_html"] = row["docketEntryText"]

            # Convert to plain text
            tree = strip_bad_html_tags_insecure(row["docketEntryText"])
            docket_text = tree.text_content()
            de["description"] = docket_text

            # "...[Entered: 09/27/2023 03:30 PM]"
            # "...[Entered: 10/05/2023 03:56 PM] [Edited: 10/06/2023 10:51 AM]"

            # It's not clear why we need to define the
            # `datetime_entered_regex` so narrowly, can't we just let
            # convert_date_string() call dateutil.parser.parse() and
            # let that fail if it doesn't work?
            # Anyhow, use the existing DATE_REGEX and then it's
            # anything goes for the time.

            # For now, skip the terminal $ anchoring to catch the Edited case.
            # We do not parse and we do ignore the Edited date.
            # # datetime_entered_regex = re.compile(
            # #     r"\[Entered:\s+(%s\s[^]]+)]$" % self.DATE_REGEX
            # # )
            datetime_entered_regex = re.compile(
                r"\[Entered:\s+(%s\s[^]]+)]" % self.DATE_REGEX
            )

            # Here's a lot of circumlocution use _get_value(), provided
            # by the superclass, which can be told to parse `date`s
            # but not `datetime`s. So we have to do it ourselves.
            datetime_str = self._get_value(datetime_entered_regex, docket_text)
            assert (
                datetime_str != ""
            ), "Docket entry's Entered: date should not be null"
            de["date_entered"] = convert_date_string(
                datetime_str, datetime=True
            )

            # Unfortunately, the server expects a `date_filed`,
            # which we don't have, and probably can't get (it's in the
            # NDA though). Although the server also doesn't know that
            # other parsers send it `date_entered` in lieu of
            # `date_filed`, without being so explicit about it. Ugh!
            de["date_filed"] = de["date_entered"]

            de["pacer_doc_id"] = row["docketEntryId"]
            de["page_count"] = row["pageCount"]

            docket_entries.append(de)

        return docket_entries


def _main():
    if len(sys.argv) != 2:
        print("Usage: python -m juriscraper.pacer.acms_docket filepath")
        print("Please provide a path to a JSON file to parse.")
        sys.exit(1)
    # Court ID is only needed for querying.
    report = ACMSDocketReport("ca9")
    filepath = sys.argv[1]
    print(f"Parsing JSON file at {filepath}")
    with open(filepath) as f:
        text = f.read()
    report._parse_text(text)
    pprint.pprint(report.data, indent=2)


if __name__ == "__main__":
    _main()



"""Scraper for Vermont Supreme Court
CourtID: vt
Court Short Name: VT
Court Contact: submit form here https://www.vermontjudiciary.org/website-feedback-form

If there are errors with the site, you can contact:

 Monica Bombard
 (802) 828-4784

She's very responsive.
"""

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.division = 7
        self.status = "Published"
        self.url = f"https://www.vermontjudiciary.org/opinions-decisions?f%5B0%5D=court_division_opinions_library_%3A{self.division}"  # supreme

    def _process_html(self):
        for case in self.html.xpath(".//article"):
            name_url_span = case.xpath(
                ".//div[contains(@class, 'views-field-name')]"
            )[0]
            date = (
                case.xpath(
                    ".//div[contains(@class, 'views-field-field-document-expiration')]"
                )[0]
                .text_content()
                .strip()
            )
            docket = (
                case.xpath(
                    ".//div[contains(@class, 'views-field-field-document-number')]"
                )[0]
                .text_content()
                .strip()
            )
            self.cases.append(
                {
                    "url": name_url_span.xpath(".//a/@href")[0],
                    "name": name_url_span.text_content(),
                    "date": date,
                    "docket": docket,
                }
            )



# Scraper for Minnesota Court of Appeals Unpublished Opinions
# CourtID: minnctapp_u
# Court Short Name: Minn Ct App
# Author: mlr
# Date: 2016-06-03


from . import minn


class Site(minn.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.parameters = {
            "v:sources": "mn-law-library-opinions",
            "query": " (url:/archive/ctapun) ",
            "sortby": "date",
        }
        self.status = "Unpublished"



from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.vacourts.gov/wpcap.htm"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath("//p"):
            links = row.xpath(".//a")
            name = row.xpath(".//b/text()")
            if not name:
                continue
            date, disposition, *_ = row.text_content().splitlines()
            date = date.split(name[0])[1]
            self.cases.append(
                {
                    "name": name[0].strip(),
                    "docket": links[0].get("href").split("/")[-1][:-4],
                    "url": links[0].get("href"),
                    "disposition": disposition,
                    "date": date.strip(),
                }
            )



"""Scraper for Colorado Appeals Court
CourtID: coloctapp
Court Short Name: Colo. Ct. App.

History:
    - 2022-01-31: Updated by William E. Palin
    - 2023-01-05: Updated by William E. Palin
    - 2023-11-04: Updated by Honey K. Joule
    - 2023-11-19: Updated by William E. Palin
"""

import datetime
import re

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = datetime.date.today().year
        self.url = f"https://www.courts.state.co.us/Courts/Court_of_Appeals/Case_Announcements/Index.cfm"
        self.status = None

    def _process_html(self):
        if self.test_mode_enabled():
            self.year = "2023"
        date = self.html.xpath("//div/p/a/text()")[0]
        for row in self.html.xpath("//p"):
            modified_string = re.sub(r"\s", "", row.text_content())
            if "PUBLISHED" == modified_string[:9]:
                self.status = "Published"
                continue
            if "UNPUBLISHED" == modified_string[:11]:
                self.status = None
                continue
            if not self.status:
                continue
            pattern = re.compile(r"\b[0-9A-Z& ]{5,}\b")
            matches = re.findall(pattern, row.text_content())
            if not matches:
                continue
            docket = matches[0].strip()
            name = row.text_content().replace(docket, "").strip()
            self.cases.append(
                {
                    "name": name,
                    "docket": docket,
                    "date": date,
                    "status": self.status,
                    "url": f"https://www.courts.state.co.us/Courts/Court_of_Appeals/Opinion/{self.year}/{docket}-PD.pdf",
                }
            )



# Scraper for California's Sixth District Court of Appeal
# CourtID: calctapp_6th
# Court Short Name: Cal. Ct. App.

from juriscraper.opinions.united_states.state import calctapp_1st


class Site(calctapp_1st.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_code = "H"
        self.division = "6th App. Dist."



from requests.exceptions import ChunkedEncodingError

from juriscraper.lib.html_utils import (
    get_row_column_links,
    get_row_column_text,
)
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://appellate-records.courts.alaska.gov/CMSPublic/Home/Opinions?isCOA=False"
        self.status = "Published"
        self.request["headers"][
            "user-agent"
        ] = "Free Law Project"  # juriscraper in the user agent crashes it - it appears to be just straight up blocked.

    def _download(self, request_dict={}):
        # Unfortunately, about 2/3 of calls are rejected by alaska but
        # if we just ignore those encoding errors we can live with it
        try:
            return super()._download(request_dict)
        except ChunkedEncodingError:
            return None

    def _process_html(self):
        if not self.html:
            return
        for table in self.html.xpath("//table"):
            date = table.xpath("./preceding-sibling::h5")[0].text_content()
            for row in table.xpath(".//tr"):
                if row.text_content().strip():
                    # skip rows without PDF links in first column
                    try:
                        url = get_row_column_links(row, 1)
                    except IndexError:
                        continue

                    self.cases.append(
                        {
                            "date": date,
                            "docket": get_row_column_text(row, 3),
                            "name": get_row_column_text(row, 4),
                            "citation": get_row_column_text(row, 2),
                            "url": url,
                        }
                    )



"""Scraper for South Carolina Supreme Court
CourtID: scctapp
Court Short Name: S.C. Ct. App.
Author: Varun Iyer
History:
 - 07-23-2018: Created.
"""

import datetime

from juriscraper.opinions.united_states.state import sc


class Site(sc.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        today = datetime.date.today()
        self.url = (
            "http://www.sccourts.org/opinions/indexCOAPub.cfm?year=%d&month=%d"
            % (today.year, today.month)
        )
        self.court_id = self.__module__
        self.court_name = "sctapp"



import re
from typing import Optional, Tuple
from urllib.parse import urljoin

import requests
from lxml.html import HtmlElement
from requests import Response

from ..lib.html_utils import (
    clean_html,
    fix_links_in_lxml_tree,
    get_html5_parsed_text,
    get_html_parsed_text,
    is_html,
    set_response_encoding,
    strip_bad_html_tags_insecure,
)
from ..lib.log_tools import make_default_logger
from .utils import (
    get_doc_id_prefix_from_court_id,
    is_pdf,
    make_doc1_url,
    make_docs1_url,
)

logger = make_default_logger()


# Patch the HtmlElement class to add a function that can handle regular
# expressions within XPath queries. See usages throughout AppellateDocketReport.
def re_xpath(self, path):
    return self.xpath(
        path, namespaces={"re": "http://exslt.org/regular-expressions"}
    )


HtmlElement.re_xpath = re_xpath


class BaseReport:
    """A base report for working with pages on PACER."""

    # Subclasses should override PATH
    PATH = ""

    # Strings below (and in subclasses) are used to identify HTML that should
    # not be parsed or processed for a variety of reasons. Spaces in the
    # strings below are converted to \s whitespace searches using regexes.
    ERROR_STRINGS = [
        "MetaMask.*web3",
        r'console\.log\(".*CloudMask',
        "Drumpf",
    ]

    def __init__(self, court_id, pacer_session=None):
        self.court_id = court_id
        self.session = pacer_session
        self.tree = None
        self.response = None
        self.is_valid = None

    @property
    def doc_id_prefix(self):
        return get_doc_id_prefix_from_court_id(self.court_id)

    @property
    def url(self):
        if self.court_id == "psc":
            return f"https://dcecf.psc.uscourts.gov/{self.PATH}"
        else:
            return f"https://ecf.{self.court_id}.uscourts.gov/{self.PATH}"

    def query(self, *args, **kwargs):
        """Query PACER and set self.response with the response."""
        raise NotImplementedError(".query() must be overridden")

    def parse(self):
        """Parse the data provided in a requests.response object and set
        self.tree to be an lxml etree. In most cases, you won't need to call
        this since it will be automatically called by self.query, if needed.

        :return: None
        """
        self.response.raise_for_status()
        set_response_encoding(self.response)
        self._parse_text(self.response.text)

    def _parse_text(self, text):
        """Parse the HTML as unicode text and set self.tree

        This is a particularly critical method when running tests, which pull
        from local disk instead of from a query response. This is also used
        when data comes from a source other than self.query() (such as a user
        upload). This method should probably be made public as .parse_text().

        :param text: A unicode object
        :return: None
        """
        assert isinstance(
            text, str
        ), f"Input must be unicode, not {type(text)}"
        text = clean_html(text)
        self.check_validity(text)
        if self.is_valid:
            self._strip_bad_html_tags_insecure(text)
            self.tree.rewrite_links(fix_links_in_lxml_tree, base_href=self.url)

    def _strip_bad_html_tags_insecure(self, text: str) -> None:
        """Remove bad tags from HTML

        :param text: The HTML string you wish to cleanup
        Set self.tree to a cleaned lxml.html.HtmlElement
        :return: None
        """
        self.tree = strip_bad_html_tags_insecure(text, remove_scripts=True)

    def check_validity(self, text: str) -> None:
        """Place sanity checks here to make sure that the returned text is
        valid and not an error page or some other kind of problem.

        Set self.is_valid flag to True or False
        """
        for error_string in self.ERROR_STRINGS:
            error_string_re = re.compile(
                r"\s+".join(error_string.split()), flags=re.I
            )
            if error_string_re.search(text):
                self.is_valid = False
                return
        self.is_valid = True

    @property
    def data(self):
        """Extract the data from the tree and return it."""
        raise NotImplementedError(".data() must be overridden.")

    def _query_pdf_download(
        self,
        pacer_case_id: str,
        pacer_doc_id: str,
        pacer_magic_num: Optional[str],
        got_receipt: str,
    ) -> Tuple[Response, str]:
        """Query the doc1 download URL.

        :param pacer_case_id: The ID of the case
        :param pacer_doc_id: The doc id for the document
        :param got_receipt: Whether to get the receipt for the page ('0') or
        get the PDF itself ('1').
        :return the Request.response object and the url queried
        """
        url = make_doc1_url(self.court_id, pacer_doc_id, True)
        data = {
            # Sending the case ID is important if you want to get PDF headers.
            # Without the case ID, PACER won't know what case it is, and won't
            # be able to add the correct headers. That'd suggest that the case
            # ID should always be sent. Unfortunately though, in many criminal
            # cases, there are documents from other related cases, and at least
            # in CourtListener, we do a bad job of keeping track of which doc
            # is from which related criminal case. If you send the *wrong* case
            # ID, you get no document at all. Though you do get a useful error
            # message. As a result, the approach we take in self.download_pdf
            # is to try it with the case ID, and then if we see that error
            # message, we try it again without the case ID. We won't get the
            # headers in that case, but we'll at least get the document.
            "caseid": pacer_case_id,
            "got_receipt": got_receipt,
            # Include the PDF header where possible. Different courts allow
            # different things here. Some have the toggle on the Docket Report
            # page to allow headers, others do not. The ones that do not *seem*
            # to default to including the headers. There's also a setting in
            # ECF accounts that you can toggle to always include headers, but
            # it's entirely unclear how it interacts with the per-docket
            # settings. Anyway, we aggressively turn on the headers without an
            # option to turn them off atm.
            "pdf_header": "1",
            "pdf_toggle_possible": "1",
        }

        # This is not always set, so we give it an option here.
        if pacer_magic_num is not None:
            data["magic_num"] = pacer_magic_num

        timeout = (60, 300)
        logger.info(f"POSTing URL: {url} with params: {data}")
        r = self.session.post(url, data=data, timeout=timeout)
        return r, url

    def download_pdf(
        self,
        pacer_case_id: str,
        pacer_doc_id: int,
        pacer_magic_num: Optional[str] = None,
        appellate: bool = False,
    ) -> Tuple[Optional[Response], str]:
        """Download a PDF from PACER.

        Note that this doesn't support attachments yet.

        :returns: A tuple of the request.Response object containing a PDF, if
        one can be found (is not sealed, gone, etc.). And a string indicating
        the error message, if there is one or else an empty string.
        """
        if pacer_magic_num:
            # If magic_number is available try to download the
            # document anonymously by its magic link

            # Create PACER base url from court_id and pacer_doc_id
            # Magic link parameters
            # We don't need the de_seq_num parameter to fetch the free document
            if appellate:
                url = make_docs1_url(self.court_id, pacer_doc_id, True)
                # For appellate documents the magic_number is the uid param
                params = {
                    "uid": pacer_magic_num,
                }
            else:
                url = make_doc1_url(self.court_id, pacer_doc_id, True)
                params = {
                    "caseid": pacer_case_id,
                    "magic_num": pacer_magic_num,
                    "use_magic": "1",  # Bypass the free look confirmation.
                }

            # Add parameters to the PACER base url and make a GET request
            req_timeout = (60, 300)
            r = requests.get(url, params=params, timeout=req_timeout)

            # If the response is an HTML document, and it doesn't contain an
            # IFRAME, the magic link document is no longer available
            error = None
            if is_html(r) and "iframe" not in r.text:
                error = (
                    f"Document not available via magic link in case: "
                    f"caseid: {pacer_case_id}, magic_num: {pacer_magic_num}, "
                    f"URL: {url}"
                )
            if error:
                logger.warning(error)
                return None, error

        else:
            # If no magic_number use normal method to fetch the document
            r, url = self._query_pdf_download(
                pacer_case_id, pacer_doc_id, pacer_magic_num, got_receipt="1"
            )

            # Use r.content instead of r.text for performance. See #564
            if b"Cannot locate the case with caseid" in r.content:
                # This document is from a different docket, but is included in
                # this docket. Probably a criminal case with the doppelganger
                # bug. Try again, but do so without the pacer_case_id.
                # This should work, but will omit the blue header on the PDFs.
                r, url = self._query_pdf_download(
                    None, pacer_doc_id, pacer_magic_num, got_receipt="1"
                )

            error = None
            if b"could not retrieve dktentry for dlsid" in r.content:
                error = (
                    f"Failed to get docket entry in case: "
                    f"{pacer_case_id=} at {url}"
                )
            if b"document is not available" in r.content:
                # See: https://ecf.akb.uscourts.gov/doc1/02211536343
                # See: https://ecf.ksd.uscourts.gov/doc1/07912639735
                # Matches against:
                # "The document is not available" and
                # "This document is not available"
                error = (
                    f"Document not available in case: "
                    f"{pacer_case_id=} at {url}"
                )
            if re.search(
                rb"You do not have permission to view\s+this document.",
                r.content,
            ):
                error = (
                    f"Permission denied getting document. It's probably "
                    f"sealed. {pacer_case_id=}, {url=}"
                )
            if b"You do not have access to this transcript." in r.content:
                error = f"Unable to get transcript. {pacer_case_id=}, {url=}"
            if b"Sealed Document" in r.content or b"Under Seal" in r.content:
                # See: https://ecf.almd.uscourts.gov/doc1/01712589088
                # See: https://ecf.cand.uscourts.gov/doc1/035122021132
                # Matches against:
                # "Sealed Document" and
                # "This document is currently Under Seal and not available..."
                error = f"Document is sealed: {pacer_case_id=} {url=}"
            if (
                b"This image is not available for viewing by non-court users"
                in r.content
            ):
                # See: https://ecf.wvsd.uscourts.gov/doc1/20115419289
                error = (
                    f"Image not available for viewing by non-court users. "
                    f"{pacer_case_id=}, {url=}"
                )
            if b"A Client Code is required for PACER search" in r.content:
                error = (
                    f"Unable to get document. Client code required: "
                    f"{pacer_case_id=}, {url=}"
                )
            if (
                b"Permission to view this document is denied based on Nature of Suit"
                in r.content
            ):
                # See: https://ecf.cacd.uscourts.gov/doc1/031134206600
                error = (
                    f"Permission denied getting document due to nature of "
                    f"suit. {pacer_case_id=}, {url=}"
                )

            if error:
                logger.warning(error)
                return None, error

            # Some pacer sites use window.location in their JS, so we have to
            # look for that. See: oknd, 13-cv-00357-JED-FHM, doc #24. But, be
            # warned, you can only catch the redirection with JS off.
            redirect_re = re.compile(rb'window\.\s*?location\s*=\s*"(.*)"\s*;')
            m = redirect_re.search(r.content)
            if m is not None:
                redirect_url = m.group(1).decode("utf-8")
                r = self.session.get(urljoin(url, redirect_url))
                r.raise_for_status()

        # The request above sometimes generates an HTML page with an iframe
        # containing the PDF, and other times returns the PDF directly. ∴
        # either get the src of the iframe and download the PDF or just return
        # the pdf.
        r.raise_for_status()
        if is_pdf(r):
            logger.info(f"Got PDF binary data for case at {url}")
            return r, ""

        text = clean_html(r.text)
        tree = get_html_parsed_text(text)
        tree.rewrite_links(fix_links_in_lxml_tree, base_href=r.url)
        try:
            iframe_src = tree.xpath("//iframe/@src")[0]
        except IndexError:
            if "pdf:Producer" in text:
                error = (
                    "Unable to download PDF. PDF content was placed "
                    f"directly in HTML. URL: {url}, caseid: {pacer_case_id}, "
                    f"magic_num: {pacer_magic_num}"
                )
            else:
                error = (
                    "Unable to download PDF. PDF not served as "
                    "binary data and unable to find iframe src "
                    f"attribute. URL: {url}, caseid: {pacer_case_id}, "
                    f"magic_num: {pacer_magic_num}"
                )
            logger.error(error)
            return None, error

        if pacer_magic_num:
            # If magic_number is available try to download the
            # document anonymously from iframe_src
            r = requests.get(iframe_src, timeout=req_timeout)
        else:
            # Use PACER session to fetch the document from iframe_src
            r = self.session.get(iframe_src)
        if is_pdf(r):
            logger.info(
                f"Got iframed PDF data for case {url} at: {iframe_src}"
            )

        return r, ""

    def is_pdf_sealed(self, pacer_case_id, pacer_doc_id, pacer_magic_num=None):
        """Check if a PDF is sealed without trying to actually download
        it.
        """
        r, url = self._query_pdf_download(
            pacer_case_id, pacer_doc_id, pacer_magic_num, got_receipt="0"
        )
        sealed = "You do not have permission to view this document."
        return sealed in r.content



"""Scraper for Court of Appeals of Kentucky
CourtID: kyctapp
Court Short Name: Ky. Ct. App.
Contact: https://courts.ky.gov/aoc/technologyservices/Pages/ContactTS.aspx

"""

import re

from juriscraper.opinions.united_states.state import ky


class Site(ky.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.parameters[
            "index"
        ] = "*{aaa711b4d9721269574a3d848abb439b} Court of Appeals Opinions (1996+)"
        self.docket_number_regex = re.compile(
            r"(?P<year>\d{4})-(?P<court>[CA]{2})-(?P<number>\d+)"
        )



from typing import List

from juriscraper.opinions.united_states.state import nytrial


class Site(nytrial.Site):
    court_regex = r"^Ct Cl|C(our)?t( [Oo]f)? Cl(aims)?$"

    def _get_child_courts(self) -> List[str]:
        """Return an empty string as child_court, since the
        New York Court of Claims in this source has no children

        :return: list of empty strings
        """
        return ["" for _ in self.cases]



"""
History:
    - 2014-08-05: Updated by mlr because it was not working, however, in middle
      of update, site appeared to change. At first there were about five
      columns in the table and scraper was failing. Soon, there were seven and
      the scraper started working without my fixing it. Very odd.
    - 2023-01-13: Update to use RSS Feed
"""

from juriscraper.opinions.united_states.federal_appellate import ca9_p


class Site(ca9_p.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.ca9.uscourts.gov/memoranda/index.xml"
        self.status = "Unpublished"



from lxml import html

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = (
            "http://www.ca10.uscourts.gov/opinions/new/daily_decisions.rss"
        )
        self.court_id = self.__module__

    def _process_html(self):
        for item in self.html.xpath(".//item"):
            for e in item.xpath(
                ".//description/text()",
                namespaces={"dc": "http://purl.org/dc/elements/1.1/"},
            ):
                if "Published Opinion" in e:
                    status = "Published"
                else:
                    status = "Unpublished"
                docket = e.split()[1].strip()
            date = convert_date_string(item.xpath(".//pubdate/text()")[0])
            formatted_date = date.strftime("%Y-%m-%d")
            self.cases.append(
                {
                    "url": html.tostring(item.xpath("link")[0], method="text")
                    .decode()
                    .replace("\\n", "")
                    .strip(),
                    "name": item.xpath(".//title/text()")[0],
                    "date": formatted_date,
                    "status": status,
                    "docket": docket,
                }
            )



"""Scraper for Connecticut Supreme Court
CourtID: conn
Court Short Name: Conn.
Author: Asadullah Baig <asadullahbeg@outlook.com>
History:
    - 2014-07-11: created
    - 2014-08-08, mlr: updated to fix InsanityError on case_dates
    - 2014-09-18, mlr: updated XPath to fix InsanityError on docket_numbers
    - 2015-06-17, mlr: made it more lenient about date formatting
    - 2016-07-21, arderyp: fixed to handle altered site format
    - 2017-01-10, arderyp: restructured to handle new format use case that includes opinions without dates and flagged for 'future' publication
    - 2022-02-02, satsuki-chan: Fixed docket and name separator, changed super class to OpinionSiteLinear
    - 2023-11-04, flooie: Fix scraper
"""

import re
from datetime import date

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = date.today().strftime("%y")
        self.url = f"http://www.jud.ct.gov/external/supapp/archiveAROsup{self.year}.htm"
        self.status = "Published"

    def _process_html(self) -> None:
        """Process the html and extract out the opinions

        :return: None
        """
        for row in self.html.xpath("//ul/preceding-sibling::p"):
            if "Published in the Law Journal" not in row.text_content():
                continue
            date = row.text_content().split(" ")[-1][:-1]
            for document in row.xpath("following-sibling::ul[1]/li"):
                output_string = re.sub(
                    r"\s{2,}", " ", document.text_content()
                ).strip()
                name = output_string.split("-")[-1].strip()
                docket = document.xpath(".//a")[0].text_content().split()[0]
                self.cases.append(
                    {
                        "date": date,
                        "url": document.xpath(".//a")[0].get("href"),
                        "docket": docket,
                        "name": name,
                    }
                )



"""Scraper for the 12th District Court of Appeals
CourtID: ohioctap_12
Court Short Name: Ohio Dist Ct App 12
Author: Andrei Chelaru
"""

from juriscraper.opinions.united_states.state import ohio


class Site(ohio.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_index = 12



# Scraper for Minnesota Supreme Court
# CourtID: minn
# Court Short Name: MN
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-03
# Contact:  Liz Reppe (Liz.Reppe@courts.state.mn.us), Jay Achenbach (jay.achenbach@state.mn.us)
import re

from juriscraper.OpinionSiteLinear import OpinionSiteLinear

# Need to contact the court, but they have a captcha on the pages that make this
# a rather unqiue scraper. I found that if i first go to the a differnet
# state website - even if the captcha triggers on that url i can still access the
# search query.


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.parameters = {
            "v:sources": "mn-law-library-opinions",
            "query": f" (url:/archive/supct) ",
            "sortby": "date",
        }
        self.url = "http://minnesota.gov/"
        self.search_url = "https://mn.gov/law-library/search/#"
        self.headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Dest": "document",
            "Accept-Language": "en-US,en;q=0.9",
            "Sec-Fetch-Mode": "navigate",
            "Host": "mn.gov",
            "User-Agent": "Juriscraper/3.0 Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.1 Safari/605.1.15",
            "Referer": "https://mn.gov/law-library/search/?v%3Asources=mn-law-library-opinions&query=+%28url%3A%2Farchive%2Fsupct%29+&citation=&qt=&sortby=&docket=&case=&v=&p=&start-date=&end-date=",
            # 'Accept-Encoding': 'gzip, deflate, br',
            "Connection": "keep-alive",
        }
        self.status = "Published"

    def _request_url_get(self, url):
        """Execute GET request and assign appropriate request dictionary
        values
        """
        self.request["session"].get(self.url, headers=self.headers, timeout=30)
        self.request["response"] = self.request["session"].get(
            self.search_url,
            params=self.parameters,
            headers=self.headers,
            timeout=30,
        )

    def _process_html(self) -> None:
        """Process the html and extract out the opinions

        :return: None
        """
        for case in self.html.xpath("//div[@class='searchresult']"):
            url = case.xpath(".//a/@href")[0]
            name = case.xpath(".//a/text()")[0]
            m = re.match(r"(.*)\. A2[23]", name)
            if m:
                name = m.groups()[0]

            summary = case.xpath(
                ".//div[@class='searchresult_snippet']/text()"
            )[0]
            raw_date = case.xpath(".//div[@class='searchresult_date']")[
                -1
            ].text_content()
            date = re.sub(r"\s+", " ", raw_date).split(":")[1].strip()
            docket = url.split("/")[-1].split("-")[0][2:]
            self.cases.append(
                {
                    "date": date,
                    "name": name,
                    "url": f"http:{url}",
                    "summary": summary,
                    "docket": docket,
                }
            )



from juriscraper.opinions.united_states.state import nytrial


class Site(nytrial.Site):
    court_regex = r"City? (Ct|Court)"
    # Most start with the regex, but there are special cases
    # such as 'Utica City Ct' in Dec 2023



import getpass
import logging.handlers
import os
import sys
import traceback

LOG_FILENAME = os.environ.get(
    "JURISCRAPER_LOG", "/var/log/juriscraper/debug.log"
)


def errprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def make_default_logger(file_path=LOG_FILENAME):
    """Boilerplate and testing code to create a logger. If we run into an
    IOError, issue a warning and use the NullHandler so things work without
    functional logging.

    :return: a logger object
    """
    logger = logging.getLogger("Logger")
    if not len(logger.handlers):
        logger.setLevel(logging.DEBUG)
        # Create a handler and attach it to the logger
        try:
            handler = logging.handlers.RotatingFileHandler(
                file_path, maxBytes=5120000, backupCount=7
            )
        except OSError as e:
            if e.errno == 2:
                errprint(
                    "\nWarning: %s: %s. "
                    "Have you created the directory for the log?"
                    % (
                        e.strerror,
                        file_path,
                    )
                )
            elif e.errno == 13:
                errprint(
                    "\nWarning: %s: %s. "
                    "Cannot access file as user: %s"
                    % (
                        e.strerror,
                        file_path,
                        getpass.getuser(),
                    )
                )
            else:
                errprint(
                    "\nIOError [%s]: %s\n%s"
                    % (e.errno, e.strerror, traceback.format_exc())
                )
            errprint(
                "Juriscraper will continue to run, and all logs will be "
                "sent to stderr."
            )
            handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(
            logging.Formatter("%(asctime)s - %(levelname)s: %(message)s")
        )
        logger.addHandler(handler)
    return logger



"""Scraper for the United States Court of Federal Claims
CourtID: uscfc
Court Short Name: Fed. Cl."""

from juriscraper.opinions.united_states.federal_special import uscfc


class Site(uscfc.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.uscfc.uscourts.gov/aggregator/sources/11"
        self.court_id = self.__module__
        self.back_scrape_iterable = list(range(1, 10))

    def _get_precedential_statuses(self):
        return ["Unpublished"] * len(self.case_names)

    def _download_backwards(self, page):
        self.url = (
            f"http://www.uscfc.uscourts.gov/aggregator/sources/11?page={page}"
        )
        self.html = self._download()



from datetime import datetime

import jinja2

from juriscraper.templates import env as jinja_env


def generate_scraper_report(file_path, results):
    no_error = []
    with_errors = []
    with_zero_results = []
    with_global_failure = []
    for name, scraper in results.items():
        has_error = False
        if scraper["global_failure"]:
            with_global_failure.append(name)
            has_error = True
        scrape = scraper["scrape"]
        if scrape.get("count") == 0:
            with_zero_results.append(name)
            has_error = True
        for exc in scrape.get("exceptions", []):
            if len(exc) > 0:
                with_errors.append(name)
                has_error = True
                break
        if not has_error:
            no_error.append(name)

    display = {
        "generated_on": datetime.now().strftime("%a, %B %d %Y %H:%M:%S"),
        "count": len(results),
        "no_error": no_error,
        "with_errors": with_errors,
        "with_zero_results": with_zero_results,
        "with_global_failure": with_global_failure,
        "total_errors": len(with_errors)
        + len(with_zero_results)
        + len(with_global_failure),
    }
    template = jinja_env.get_template(
        "juriscraper/templates/report.html.jinja2"
    )
    with open(file_path, "w") as f:
        f.write(template.render(display))



"""
Scraper for Florida 5th District Court of Appeal
CourtID: flaapp5
Court Short Name: flaapp5
Court Contact: 5dca@flcourts.org, 386-947-1530
Author: Andrei Chelaru
Reviewer: mlr
History:
 - 2014-07-23, Andrei Chelaru: Created.
 - 2014-08-05, mlr: Updated.
 - 2014-08-06, mlr: Updated.
 - 2014-09-18, mlr: Updated date parsing code to handle Sept.
 - 2016-03-16, arderyp: Updated to return proper absolute pdf url paths, simplify date logic
"""

import re

from lxml import html

from juriscraper.lib.string_utils import (
    clean_string,
    convert_date_string,
    normalize_dashes,
)
from juriscraper.OpinionSite import OpinionSite

from . import fladistctapp


class Site(fladistctapp.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.number = "fifth"
        self.base = "https://5dca.flcourts.gov"
        self.update_url()



# Base scraper
# CourtID: flaapp
# Court Short Name: flaapp

from datetime import datetime, timedelta

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_number = None
        self.offset = 0
        self.number = None
        self.base = "https://1dca.flcourts.gov"
        self.update_url()

    def update_url(self):
        """"""
        today = datetime.now().strftime("%m/%d/%Y")
        prev = (datetime.now() - timedelta(days=7)).strftime("%m/%d/%Y")

        self.url = f"{self.base}/search?sort=opinion/disposition_date%20desc,%20opinion/case_number%20asc&view=full&searchtype=opinions&limit=10&scopes[]={self.number}_district_court_of_appeal&type[]=pca&type[]=written&startdate={prev}&enddate={today}&date[year]=&date[month]=&date[day]=&query=&offset={self.offset}"

    def _process_html(self) -> None:
        """Process the html and extract out the opinions

        :return: None
        """
        for row in self.html.xpath("//tr"):
            if not len(row.xpath(".//td")):
                continue
            (
                link,
                filetype,
                docket,
                name,
                disposition,
                note,
                date_filed,
            ) = row.xpath(".//td")
            self.cases.append(
                {
                    "name": name.text_content().strip(),
                    "date": date_filed.text_content().strip(),
                    "disposition": disposition.text_content().strip(),
                    "url": link.xpath(".//a")[0].get("href"),
                    "status": "Published",
                    "docket": docket.text_content().strip(),
                }
            )

        if not self.html.xpath('.//li[@class="next disabled"]'):
            # If pagniation continues loop
            self.offset = self.offset + 10
            self.update_url()
            self.html = super()._download()
            self._process_html()



"""Scraper for Superior Court of Maine
CourtID: mesuperct
Court Short Name: Me. Super Ct
Author: William E. Palin
Date created: December 28, 2023

History:
  2023-12-28: Created
"""

from datetime import date

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = f"https://apps.maine.edu/SuperiorCourt/show_list.jsp?plaintiff=&defendant=&year={date.today().year}&code=&rule=&title=&number=&section=&Search=Search"
        self.base_path = ["placeholder"]
        self.back_scrape_iterable = [1]

    def _download_backwards(self, d: date) -> None:
        """Sets the URL for the backwards download based on the given date.

        :param d: Just a placeholder value
        :return: None
        """
        self.url = "https://apps.maine.edu/SuperiorCourt/show_list.jsp?plaintiff=&defendant=&year=&code=&rule=&title=&number=&section=&Search=Search"

    def _process_html(self) -> None:
        """Processes the HTML content and extracts case information.

        Iterates over the rows of the HTML table, extracting case details and appending
        them to the cases list as dictionaries.

        :return: None
        """

        for row in self.html.xpath("//tr")[1:]:
            docket_number = row.xpath(".//td//a/text()")[0].strip()
            date = row.xpath(".//td/text()")[-1].strip()
            plaintiff = row.xpath(".//td/a/text()")[1].strip()
            defendant = row.xpath(".//td/a/text()")[2].strip()
            url = f"https://files.mainelaw.maine.edu/library/SuperiorCourt/decisions/{docket_number}.pdf"
            author = row.xpath(".//td/a/text()")[3].strip()
            self.cases.append(
                {
                    "date": date,
                    "docket": docket_number,
                    "url": url,
                    "name": f"{plaintiff} v. {defendant}",
                    "status": "Unpublished",
                    "judge": author,
                }
            )



# Author: Michael Lissner
# Date created: 2013-05-23


from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = (
            "https://www.courts.state.hi.us/opinions_and_orders/opinions"
        )
        self.court_id = self.__module__
        self.court_code = "S.Ct"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath("//tr[@class='row-']"):
            date, court, docket, name, lower_court, citation = row.xpath(
                ".//td"
            )
            name = name.text_content().split("(")[0]
            court = court.text_content()
            if court != self.court_code:
                continue
            self.cases.append(
                {
                    "date": date.text_content(),
                    "name": name,
                    "docket": docket.text_content()
                    .strip()
                    .split("\t")[0]
                    .split()[0],
                    "url": docket.xpath(".//a")[0].get("href"),
                    "lower_courts": lower_court.text_content(),
                    "citation": citation.text_content(),
                }
            )



"""Scraper for Wyoming Supreme Court
CourtID: wyo
Court Short Name: Wyo.
History:
 - 2014-07-02: mlr: Created new version when court got new website.
 - 2015-07-06: m4h7: Updated to use JSON!
 - 2016-06-09: arderyp: Updated because json endpoint moved and was changed
"""

import re
from datetime import date, datetime

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.base_url = "http://www.courts.state.wy.us"
        self.download_base = "https://documents.courts.state.wy.us/Opinions"
        self.url = (
            "https://opinions.courts.state.wy.us/Home/GetOpinions?StartDate=1%2F1%2F"
            + str(date.today().year)
        )

    def _get_case_names(self):
        return [
            f"{opinion['Appellant']} v. {opinion['Appellee']}"
            for opinion in self.html
        ]

    def _get_download_urls(self):
        download_urls = []
        for record in self.html:
            pdf_file_name = record["DocumentName"]
            if pdf_file_name[:5] == "../..":
                pdf_file_name = pdf_file_name[5:]
            url = f"{self.download_base}/{pdf_file_name}".replace(" ", "%20")
            download_urls.append(url)
        return download_urls

    def _get_case_dates(self):
        case_dates = []
        date_re = re.compile(r"^/Date\((\d+)\)/$")
        for record in self.html:
            match = date_re.match(record["date_heard"])
            if match:
                timestamp = int(match.group(1)) / 1000
                case_dates.append(datetime.fromtimestamp(timestamp).date())
        return case_dates

    def _get_docket_numbers(self):
        return [opinion["DocketNumber"] for opinion in self.html]

    def _get_citations(self):
        return [opinion["OpinionID"] for opinion in self.html]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)



# Scraper for Supreme Court of Oklahoma
# CourtID: okla
# Court Short Name: OK
# Court Contact: webmaster@oscn.net
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-05


from datetime import datetime

from lxml import html

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        year = datetime.today().year
        self.url = f"https://www.oscn.net/applications/oscn/Index.asp?ftdb=STOKCSSC&year={year}&level=1"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath("//div/p['@class=document']")[::-1]:
            if "OK" not in row.text_content():
                continue
            if "EMAIL" in row.text_content():
                continue
            if "P.3d" in row.text_content():
                citation1, citation2, date, name = row.text_content().split(
                    ",", 3
                )
                citation = f"{citation1} {citation2}"
            else:
                citation, date, name = row.text_content().split(",", 2)

            self.cases.append(
                {
                    "date": date,
                    "name": name,
                    "docket": citation,
                    "url": row.xpath(".//a")[0].get("href"),
                    "citation": citation,
                }
            )

    @staticmethod
    def cleanup_content(content):
        print(content)
        tree = html.fromstring(content)
        core_element = tree.xpath("//*[@id='oscn-content']")[0]
        return html.tostring(
            core_element, pretty_print=True, encoding="unicode"
        ).encode("utf-8")



"""Backscraper for Board of Immigration Appeals
CourtID: bia
Court Short Name: BIA.
Author: William E. Palin
Date created: December 27, 2021
"""

from juriscraper.opinions.united_states.administrative_agency import bia


class Site(bia.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.back_scrape_iterable = range(0, 21)

    def _download_backwards(self, volume: int) -> None:
        """Download backwards over the volume links in the DOJ BIA website.

        :param volume: The volume index for the URL
        :return: None
        """
        self.volume = volume
        if not self.urls:
            self.html = self._download()
        self._process_html()



"""Scraper for the Navy-Marine Corps Court of Criminal Appeals
CourtID: nmcca
Court Short Name:
Reviewer: mlr
History:
    15 Sep 2014: Created by Jon Andersen
"""

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = (
            "https://www.jag.navy.mil/api/tables/decisions-opinions/data/"
        )
        self.court_id = self.__module__
        self.request["verify"] = False

    def _process_html(self):
        for row in self.html["results"]:
            date, docket, notes, name = list(row["data"].values())
            url = row["documents"][0]["document"]["download_url"]
            if notes == "Unpublished":
                status = "Unpublished"
            else:
                status = "Published"
            self.cases.append(
                {
                    "date": date,
                    "name": name,
                    "url": url,
                    "status": status,
                    "docket": docket,
                }
            )



"""Scraper for Colorado Supreme Court
CourtID: colo
Court Short Name: Colo.
Author: Philip Ardery
Reviewer: mlr
Date created: 2016-06-03
History:
    - 2022-01-31: Updated by William E. Palin
    - 2023-01-05: Updated by WEP
    - 2023-11-19: Drop Selenium by WEP
    - 2023-12-20: Updated with citations, judges and summaries, Palin
"""
import datetime
import re
from datetime import date, timedelta
from typing import Any, Dict

from dateutil import parser

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Published"
        self.url = "https://www.courts.state.co.us/Courts/Supreme_Court/Proceedings/Index.cfm"

    def _process_html(self):
        """"""
        for row in self.html.xpath("//div[@id='Dispositions']/a"):
            case_id = row.attrib["onclick"].split("'")[1]
            div = row.xpath(f"//div[@id='Case_{case_id}']")[0]
            if not div.xpath(".//a/text()"):
                # This is set to avoid data decades back
                continue
            document_type = div.xpath(".//a/text()")[0]
            if "Opinion" not in document_type:
                # Only collect opinions and not orders
                continue
            summary = (
                row.xpath(f"//div[@id='Case_{case_id}']")[0]
                .text_content()
                .strip()
            )
            url = div.xpath(".//a/@href")[0]
            title = row.xpath("following-sibling::text()")[0].strip()
            docket, name = title.split(" ", 1)
            name, _ = name.split("  ")
            if "(Honorable" in name:
                name, judge = name.split("(")
                name = name.strip()
                judges = judge.strip(")").strip().replace("Honorable", "")
            else:
                judges = ""
            date_filed = self.find_date(summary)
            if parser.parse(date_filed).date() < self.set_min_date():
                # Only collect back 6 months
                break
            if "https://www.courts.state.co.us/" not in url:
                url = f"https://www.courts.state.co.us/{url}"
            self.cases.append(
                {
                    "summary": summary,
                    "date": date_filed,
                    "name": name,
                    "docket": docket.strip(","),
                    "url": url,
                    "judge": judges,
                }
            )

    def set_min_date(self):
        """Set minimum date to add opinions

        :return: Date 6 months back
        """
        if self.test_mode_enabled():
            today = datetime.date(2023, 11, 19)
            return today - timedelta(180)
        else:
            return date.today() - timedelta(180)

    def find_date(self, summary) -> str:
        """Find date filed

        Normally it follows a typical pattern but not always

        :param summary: Use summary text to find date filed
        :return: date as string
        """
        if "Opinion issued" in summary:
            return summary.split("Opinion issued")[1]
        date_pattern = re.compile(
            r"((January|February|March|April|May|June|July|August|September|October|November|December)\s+(\d{1,2})\s?,?\s+(\d{4}))"
        )
        match = re.findall(date_pattern, summary)
        date_filed = match[-1][0] if match else ""
        return date_filed

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Extract Citation from text

        :param scraped_text: Text of scraped content
        :return: date filed
        """
        m = re.findall(r"(20\d{2})\s(CO)\s(\d+A?)", scraped_text)
        if m:
            vol, reporter, page = m[0]
            return {
                "Citation": {
                    "volume": vol,
                    "reporter": reporter,
                    "page": page,
                    "type": 8,  # NEUTRAL in courtlistener Citation model
                },
            }
        return {}



from .acms_docket import ACMSDocketReport
from .appellate_attachment_page import AppellateAttachmentPage
from .appellate_docket import AppellateDocketReport
from .attachment_page import AttachmentPage
from .case_query import CaseQuery
from .case_query_advanced import CaseQueryAdvancedBankruptcy
from .claims_activity import ClaimsActivity
from .claims_register import ClaimsRegister
from .docket_history_report import DocketHistoryReport
from .docket_report import DocketReport
from .download_confirmation_page import DownloadConfirmationPage
from .email import NotificationEmail, S3NotificationEmail
from .free_documents import FreeOpinionReport
from .hidden_api import PossibleCaseNumberApi, ShowCaseDocApi
from .http import PacerSession
from .internet_archive import InternetArchive
from .list_of_creditors import ListOfCreditors
from .mobile_query import MobileQuery
from .rss_feeds import PacerRssFeed

__all__ = [
    ACMSDocketReport,
    AppellateDocketReport,
    AttachmentPage,
    AppellateAttachmentPage,
    CaseQuery,
    CaseQueryAdvancedBankruptcy,
    ClaimsActivity,
    ClaimsRegister,
    DocketHistoryReport,
    DocketReport,
    DownloadConfirmationPage,
    FreeOpinionReport,
    InternetArchive,
    ListOfCreditors,
    NotificationEmail,
    S3NotificationEmail,
    PacerRssFeed,
    PacerSession,
    PossibleCaseNumberApi,
    ShowCaseDocApi,
]



"""Scraper for First Circuit of Appeals
CourtID: ca1
Court Short Name: ca1
Author: Michael Lissner
Date created: 13 June 2014
History:
- 2014-06-13: Created by mlr
- 2022-09-20: Updated by flooie
"""


import feedparser

from juriscraper.OralArgumentSiteLinear import OralArgumentSiteLinear


class Site(OralArgumentSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__

        self.url = "http://www.ca1.uscourts.gov/doarrss/feed"

    def _process_html(self) -> None:
        """Process the RSS feed.

        Iterate over each item in the RSS feed to extract out
        the date, case name, docket number, and status and pdf URL.
        Return: None
        """
        feed = feedparser.parse(self.request["response"].content)
        for item in feed["entries"]:
            self.cases.append(
                {
                    "date": item["published"].split()[0],
                    "url": item["link"],
                    "status": "Unknown",
                    "docket": item["id"].split(" ", 2)[1].strip(","),
                    "name": item["id"].split(" ", 2)[2],
                }
            )



from juriscraper.AbstractSite import logger


class DeferringList:
    """This object can be used to do deferred loading of meta data in the case
    that a piece of meta data requires some special work to obtain.

    Note that since this inherits from object (rather than list), it won't be
    sorted by the _date_sort function. As a result, it's vital that the code
    using this object provide the seed data in sorted order. Failure to do so
    will result in mixed up data being sent to the caller -- a bad fate.

    For an example of how this can be used, see
    juriscraper.opinions.united_states.state.tex
    """

    def __init__(self, *args, **kwargs):
        logger.warning(
            "Using DeferringList object which cannot be sorted until "
            "fetched. Note that in usual processing, the fetching "
            "happens before the sorting, so this is OK."
        )
        logger.info(
            f"DeferringList has {len(kwargs['seed'])} entries to fetch."
        )
        self._data = kwargs["seed"]
        self._fetched_items = [False] * len(kwargs["seed"])
        self._fetching_function = kwargs["fetcher"]

    def __iter__(self):
        for item in range(0, len(self._data)):
            if self._fetched_items[item]:
                yield self._data[item]
            else:
                yield self.__getitem__(item)

    def __getitem__(self, item):
        if self._fetched_items[item]:
            return self._data[item]
        else:
            # Go get the item using the fetching function
            logger.info(
                f"Getting deferred value from seed: {self._data[item]}"
            )
            new_val = self._fetching_function(self._data[item])
            self._data[item] = new_val
            self._fetched_items[item] = True
            return new_val

    def __setitem__(self, key, value):
        if self._fetched_items[key]:
            self._data[key] = value
        else:
            raise AttributeError(
                "Cannot set item that has not yet been fetched."
            )

    def __delitem__(self, item):
        del self._data[item]
        del self._fetched_items[item]

    def __len__(self):
        return len(self._data)

    def __str__(self):
        return f"<DeferringList {self.__dict__}>"



"""Scraper for Supreme Court of U.S.
CourtID: scotus
Court Short Name: scotus
History:
 - 2014-07-20 - Created by Andrei Chelaru, reviewed by MLR
 - 2017-10-09 - Updated by MLR.
"""

from datetime import datetime

from juriscraper.OralArgumentSite import OralArgumentSite


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "http://www.supremecourt.gov/oral_arguments/argument_audio.aspx"
        )
        self.back_scrape_iterable = list(range(2010, 2015))

    def _get_download_urls(self):
        path = "id('list')//tr//a/text()"
        return list(map(self._return_download_url, self.html.xpath(path)))

    @staticmethod
    def _return_download_url(d):
        file_type = "mp3"  # or 'wma' is also available for any case.
        download_url = "http://www.supremecourt.gov/media/audio/{type}files/{docket_number}.{type}".format(
            type=file_type, docket_number=d
        )
        return download_url

    def _get_case_names(self):
        path = "id('list')//tr/td/span/text()"
        return [s.lstrip(". ") for s in self.html.xpath(path)]

    def _get_case_dates(self):
        path = "id('list')//tr/td[2]//text()"
        return [
            datetime.strptime(s, "%m/%d/%y").date()
            for s in self.html.xpath(path)
            if not "Date" in s
        ]

    def _get_docket_numbers(self):
        path = "id('list')//tr//a/text()"
        return list(self.html.xpath(path))

    def _download_backwards(self, year):
        self.url = (
            f"http://www.supremecourt.gov/oral_arguments/argument_audio/{year}"
        )
        self.html = self._download()



import time
from datetime import date

from lxml import html

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.cadc.uscourts.gov/internet/opinions.nsf/uscadcopinions.xml"
        self.court_id = self.__module__

    def _get_case_names(self):
        return [e for e in self.html.xpath("//item/description/text()")]

    def _get_download_urls(self):
        return [
            html.tostring(e, method="text").decode()
            for e in self.html.xpath("//item/link")
        ]

    def _get_case_dates(self):
        dates = []
        for date_string in self.html.xpath("//item/pubdate/text()"):
            date_only = " ".join(date_string.split(" ")[1:4])
            dates.append(
                date.fromtimestamp(
                    time.mktime(time.strptime(date_only, "%d %b %Y"))
                )
            )
        return dates

    def _get_docket_numbers(self):
        return [
            e.split("|")[0] for e in self.html.xpath("//item/title/text()")
        ]

    def _get_precedential_statuses(self):
        return ["Published" for _ in range(0, len(self.case_names))]



"""Scraper for the Air Force Court of Criminal Appeals
CourtID: afcca
Court Short Name: Air Force Court of Criminal Appeals
Reviewer: mlr
History:
    15 Sep 2014: Created by Jon Andersen
"""

from datetime import date

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url_base = "http://afcca.law.af.mil/content/opinions_date_%d.html"
        self.url = self.url_base % date.today().year
        self.disable_certificate_verification()

    def _process_html(self):
        for row in self.html.xpath(".//img[contains(@src, 'pdf.gif')]/../..")[
            :-1
        ]:
            status = row.xpath(".//td")[3].text_content()
            if status != "Published":
                status = "Unpublished"
            self.cases.append(
                {
                    "url": row.xpath(".//td/a/@href")[0],
                    "docket": row.xpath(".//td")[2].text_content(),
                    "name": row.xpath(".//td/a")[0].text_content(),
                    "date": row.xpath(".//td")[4].text_content(),
                    "status": status,
                }
            )



# Court Contact: bkraft@courts.ms.gov (see https://courts.ms.gov/aoc/aoc.php)

import datetime

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


# Landing page: https://courts.ms.gov/appellatecourts/sc/scdecisions.php
class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain = "https://courts.ms.gov"
        self.court_id = self.__module__
        self.method = "POST"
        self.number_of_dates_to_process = 5
        self.pages = {}
        self.parameters = {"crt": self.get_court_parameter()}
        self.status = "Published"
        self.url = f"{self.domain}/appellatecourts/docket/gethddates.php"

    def get_court_parameter(self):
        return "SCT"

    """Retrieve dates for which there are case listings.
    This site's architecture is no bueno. We have to issue
    a POST request to this page to get a array (in the form
    of a string) or dates that have cases associated with
    them.
    """

    def _download(self, request_dict={}):
        dates_page = super()._download(request_dict)
        self.parse_date_pages(dates_page)

    """Keep track of the most recent N date pages.
    We dont want to crawl all the way back to 1996, so we only
    parse the most recent [self.number_of_dates_to_process]
    number of date pages.  Since cases are usually published
    once a week, this means scraping about the most recent
    months worth of cases.
    """

    def parse_date_pages(self, dates_page):
        # For testing, each example file should be a specific sub-date page,
        # like https://courts.ms.gov/Images/HDList/SCT02-27-2020.html
        if self.test_mode_enabled():
            # date below is arbitrary and doesnt matter, it just
            # needs to be static for testing to work
            self.pages["2020-02-28"] = dates_page
            return
        for date in self.get_dates_from_date_page(dates_page):
            url = "{}/Images/HDList/SCT{}.html".format(
                self.domain,
                datetime.date.strftime(date, "%m-%d-%Y"),
            )
            page = self._get_html_tree_by_url(url)
            self.pages[f"{date}"] = page

    """Convert string of dates on page into list of date objects.
    """

    def get_dates_from_date_page(self, dates_page):
        dates = []
        substrings = dates_page.text_content().split('"')
        for substring in substrings:
            try:
                dates.append(convert_date_string(substring))
            except ValueError:
                pass
        dates.sort(reverse=True)
        return dates[: self.number_of_dates_to_process]

    def _process_html(self):
        for date, page in self.pages.items():
            for anchor in page.xpath(".//a[contains(./@href, '.pdf')]"):
                parent = anchor.getparent()

                # sometimes the first opinion on the pages is nested
                # in a <p> tag for whatever reason.
                while parent.getparent().tag != "body":
                    parent = parent.getparent()

                sections = parent.xpath("./following-sibling::ul")
                if not sections:
                    # the while loop above should mean we never fall in here
                    continue

                section = sections[0]
                self.cases.append(
                    {
                        "date": date,
                        "docket": anchor.text_content().strip(),
                        "name": section.xpath(".//b")[0]
                        .text_content()
                        .strip(),
                        "summary": section.text_content().strip(),
                        "url": anchor.xpath("./@href")[0],
                    }
                )



"""
Scraper for Indiana Supreme Court
CourtID: ind
Court Short Name: Ind.
Auth: Jon Andersen <janderse@gmail.com>
Reviewer: mlr
History:
    2014-09-03: Created by Jon Andersen
"""
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_name = "Supreme Court"
        self.status = "Published"
        self.url = "https://public.courts.in.gov/Decisions/api/Search"

    def _download(self):
        if self.test_mode_enabled():
            import json

            return json.load(open(self.url))
        return (
            self.request["session"]
            .get(
                self.url,
                headers=self.request["headers"],
                verify=self.request["verify"],
                timeout=60,
            )
            .json()
        )

    def _process_html(self):
        for case in self.html:
            if case["courtName"] != self.court_name:
                continue
            self.cases.append(
                {
                    "name": case["style"],
                    "docket": case["caseNumber"],
                    "date": case["date"],
                    "url": f"https://public.courts.in.gov/Decisions/{case['opinionUrl']}",
                    "disposition": case["decision"],
                }
            )



"""
Backscraper for Indiana Supreme Court
CourtID: ind
Court Short Name: Ind.
Auth: Mike Lissner <mike@freelawproject.org>
Reviewer:
History:
    2014-09-02: Bug fix by Jon Andersen - recognize "(NFP)" in case name
    2014-09-03: Moved to backscrapers
"""
import time
from datetime import date

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.in.gov/judiciary/opinions/archsup.html"
        self.court_id = self.__module__
        self.my_precedential_statuses = []

    def _get_case_names(self):
        raw_case_names = [
            s
            for s in self.html.xpath(
                "//table/tr/td[3]/table/tr[4]//table/tr[position() > 1]/td[2]//a/text()"
            )
        ]
        case_names = []
        self.my_precedential_statuses = []
        for case_name in raw_case_names:
            if case_name.find("(NFP)") >= 0:
                case_names.append(case_name.replace("(NFP)", "").strip())
                self.my_precedential_statuses.append("Unpublished")
            else:
                case_names.append(case_name)
                self.my_precedential_statuses.append("Published")
        return case_names

    def _get_download_urls(self):
        return [
            s
            for s in self.html.xpath(
                "//table/tr/td[3]/table/tr[4]//table/tr[position() > 1]/td[2]//a/@href"
            )
        ]

    def _get_case_dates(self):
        dates = []
        for date_string in self.html.xpath(
            "//table/tr/td[3]/table/tr[4]//table/tr[position() > 1]/td[1]//font/text()"
        ):
            val = date_string.strip()
            if val == "":
                dates.append("")
            else:
                dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime(val, "%m/%d/%y"))
                    )
                )
        return dates

    def _get_docket_numbers(self):
        return [
            s
            for s in self.html.xpath(
                "//table/tr/td[3]/table/tr[4]//table/tr[position() > 1]/td[4]//font/text()"
            )
        ]

    def _get_lower_court_numbers(self):
        return [
            e if e.strip() != "N/A" else ""
            for e in self.html.xpath(
                "//table/tr/td[3]/table/tr[4]//table/tr[position() > 1]/td[3]//font/text()"
            )
        ]

    def _get_precedential_statuses(self):
        return self.my_precedential_statuses



"""Scraper for Seventh Circuit of Appeals
CourtID: ca7
Court Short Name: ca7
Author: Andrei Chelaru
Reviewer: mlr
Date created: 19 July 2014
"""

from datetime import datetime

from juriscraper.OralArgumentSite import OralArgumentSite


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://media.ca7.uscourts.gov/oralArguments/oar.jsp?caseyear=&casenumber=&period=Past+month"

    def _get_download_urls(self):
        path = "//tr[@bgcolor]/td[5]//@href"
        return list(self.html.xpath(path))

    def _get_case_names(self):
        path = "//tr[@bgcolor]/td[2]/text()"
        return list(self.html.xpath(path))

    def _get_case_dates(self):
        path = "//tr[@bgcolor]/td[4]/text()"
        return list(map(self._return_case_date, self.html.xpath(path)))

    @staticmethod
    def _return_case_date(s):
        return datetime.strptime(s.strip(), "%m/%d/%Y").date()

    def _get_docket_numbers(self):
        path = "//tr[@bgcolor]/td[1]/text()"
        return list(self.html.xpath(path))



"""Scraper and Back Scraper for New York Commercial Division
CourtID: nysupct_commercial
Court Short Name: NY
History:
 - 2024-01-05, grossir: modified to use nytrial template
"""
from datetime import date

from juriscraper.opinions.united_states.state import nytrial


class Site(nytrial.Site):
    base_url = "https://nycourts.gov/reporter/slipidx/com_div_idxtable.shtml"
    court_regex = r".*"
    first_opinion_date = date(2013, 7, 1)



"""Scraper for Second Circuit
CourtID: ca2
Contact:
  Webmaster@ca2.uscourts.gov ('Calendar Team' handles updates, and is responsive)
  ecfhelpdesk@ca2.uscourts.gov
  Shane_Clouden@ca2.uscourts.gov
"""


import time
from datetime import date, timedelta

from dateutil.rrule import DAILY, rrule
from lxml import html

from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.interval = 30
        self.url = "http://www.ca2.uscourts.gov/decisions?IW_DATABASE=OPN&IW_FIELD_TEXT=*&IW_SORT=-Date"
        self.court_id = self.__module__
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=self.interval,
                dtstart=date(2007, 1, 1),
                until=date(2015, 1, 1),
            )
        ]

    def _get_case_names(self):
        return [titlecase(t) for t in self.html.xpath("//table/td[2]/text()")]

    def _get_download_urls(self):
        return [e for e in self.html.xpath("//table/td/b/a/@href")]

    def _get_case_dates(self):
        dates = []
        for e in self.html.xpath("//table/td[3]/text()"):
            dates.append(
                date.fromtimestamp(time.mktime(time.strptime(e, "%m-%d-%Y")))
            )
        return dates

    def _get_docket_numbers(self):
        return [
            e.text_content() for e in self.html.xpath("//table/td/b/a/nobr")
        ]

    def _get_precedential_statuses(self):
        statuses = []
        for status in self.html.xpath("//table/td[4]/text()"):
            if "opn" in status.lower():
                statuses.append("Published")
            elif "sum" in status.lower():
                statuses.append("Unpublished")
            else:
                statuses.append("Unknown")
        return statuses

    def _download_backwards(self, d):
        self.url = "http://www.ca2.uscourts.gov/decisions?IW_DATABASE=OPN&IW_FIELD_TEXT=*&IW_SORT=-Date&IW_BATCHSIZE=100&IW_FILTER_DATE_BEFORE={before}&IW_FILTER_DATE_After={after}".format(
            before=(d + timedelta(self.interval)).strftime("%Y%m%d"),
            after=d.strftime("%Y%m%d"),
        )
        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200



"""Scraper for Federal Circuit of Appeals
CourtID: cafc
Court Short Name: cafc
Author: Andrei Chelaru
Reviewer: mlr
History:
 - created by Andrei Chelaru, 18 July 2014
 - Updated/rewritten by mlr, 2016-04-14
 - Updated by William E. Palin, 2022-05-17
 - Updated by William E. Palin, 2023-01-26
"""
from datetime import date, timedelta

from dateutil.rrule import MONTHLY, rrule

from juriscraper.OralArgumentSiteLinear import OralArgumentSiteLinear


class Site(OralArgumentSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://cafc.uscourts.gov/home/oral-argument/listen-to-oral-arguments/"
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                MONTHLY,
                dtstart=date(2021, 10, 8),
                until=date(2022, 5, 17),
            )
        ]

    def _download(self, request_dict={}):
        """Download from cafc

        :param request_dict: Nothing
        :return: none
        """
        if self.method == "GET":
            return super()._download(request_dict)
        else:
            self.html = super()._download(request_dict)

    def _set_parameters(self) -> None:
        """Set the parameters for the request.

        :return:None
        """
        self.parameters = {
            "draw": "4",
            "columns[0][data]": "0",
            "columns[0][name]": "Arg_Date",
            "columns[0][searchable]": "true",
            "columns[0][orderable]": "true",
            "columns[0][search][regex]": "false",
            "columns[1][data]": "1",
            "columns[1][name]": "Appeal_Number",
            "columns[1][searchable]": "true",
            "columns[1][orderable]": "true",
            "columns[1][search][value]": "",
            "columns[1][search][regex]": "false",
            "columns[2][data]": "2",
            "columns[2][name]": "Arg_Link",
            "columns[2][searchable]": "true",
            "columns[2][orderable]": "true",
            "columns[2][search][value]": "",
            "columns[2][search][regex]": "false",
            "order[0][column]": "0",
            "order[0][dir]": "desc",
            "start": "0",
            "length": "1000",
            "search[value]": "",
            "search[regex]": "false",
            "wdtNonce": self.html.xpath(
                ".//input[@id='wdtNonceFrontendEdit_8']/@value"
            ),
            "sRangeSeparator": "|",
        }
        self.today = date.today().strftime("%m/%d/%Y")
        self.earlier = (date.today() - timedelta(days=30)).strftime("%m/%d/%Y")
        self.parameters[
            "columns[0][search][value]"
        ] = f"{self.earlier}|{self.today}"

    def _fetch_json(self):
        """Update parameters and fetch json

        Switch to POST set params and download JSON

        :return: None
        """
        # Fetch the JSON api endpoint
        self.method = "POST"
        self._set_parameters()
        self.url = "https://cafc.uscourts.gov/wp-admin/admin-ajax.php?action=get_wdtable&table_id=8"
        self._download()

    def _process_html(self) -> None:
        """Extract content from JSON response

        Each row in the json returns a list of values, see example below.
        [
          "05/06/2022",
          "21-2084",
          "<a sort=\"Best Medical International, Inc. v. Elekta Inc.\"
          href=\"https://oralarguments.cafc.uscourts.gov/default.aspx?fl=21-2084_05062022.mp3\"
          download>Best Medical International, Inc. v. Elekta Inc. (mp3)</a>"
        ]
        :return: None
        """
        if not self.test_mode_enabled():
            self._fetch_json()
        for row in self.request["response"].json()["data"]:
            _, name, _, url, _ = row[2].split('"')
            self.cases.append(
                {"date": row[0], "docket": row[1], "name": name, "url": url}
            )

    def _download_backwards(self, d: date) -> None:
        """Download a months' worth of oral arguments.

        :param d: Date to download arguments starting from
        :return: None
        """
        self.end = (d + timedelta(days=31)).strftime("%m/%d/%Y")
        self.parameters[
            "columns[0][search][value]"
        ] = f'{d.strftime("%m/%d/%Y")}|{self.end}'
        self.html = self._download()
        self._process_html()



"""Scraper for COURT OF APPEALS OF THE STATE OF NEVADA
CourtID: nevapp

History:
    - 2023-12-13: Created by William E. Palin
"""
from juriscraper.opinions.united_states.state import nev


class Site(nev.Site):
    def _process_html(self):
        for case in self.html:
            if "COA" not in case["caseNumber"]:
                continue
            self.cases.append(
                {
                    "name": case["caseTitle"],
                    "docket": case["caseNumber"],
                    "date": case["date"],
                    "url": case["docurl"],
                }
            )



"""
Scraper for Pennsylvania Superior Court
CourtID: pasup
Court Short Name: pasup
Author: Andrei Chelaru
Reviewer: mlr
Date created: 21 July 2014
"""

from juriscraper.opinions.united_states.state import pa


class Site(pa.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.pacourts.us/Rss/Opinions/Superior/"



from juriscraper.opinions.united_states.state import idaho_civil


class Site(idaho_civil.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.isc.idaho.gov/appeals-court/coa_criminal"
        self.court_id = self.__module__



"""
Scraper for the United States Bankruptcy Appellate Panel for the First Circuit
CourtID: bap1
Court Short Name: 1st Cir. BAP
Court Contact: ca01_BAP@ca1.uscourts.gov, (617) 748-9650
Author: Gianfranco Rossi
History:
 - 2023-12-28, grossir: created
"""
import calendar
import re
from typing import Any, Dict, Optional

from lxml.html import HtmlElement

from juriscraper.lib.exceptions import SkipRowError
from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    lower_court_to_abbreviation = {
        "USBC - District of New Hampshire": "NH",
        "USBC - District of Massachusetts (Worcester)": "MW",
        "USBC - District of Puerto Rico": "PR",
        "USBC - District of Massachusetts (Boston)": "MB",
        "USBC - District of Maine (Portland)": "EP",
        "USBC - District of Rhode Island": "RI",
        "Bankrupcty Court of ME - Bangor": "EB",
        "Bankruptcy Court of MA - Boston": "MB",
        "Bankruptcy Court of MA - Springfield": "MS",
        "Bankruptcy Court of ME - Portland": "EP",
        "Bankruptcy Court - Rhode Island": "RI",
        "Bankruptcy Court - San Juan Puerto Rico": "PR",
        "Bankruptcy Court of MA - Worcester": "MW",
        "Bankruptcy Court - Ponce Puerto Rico": "PR",
        "Bankruptcy Court of NH, Concord": "NH",
    }
    # See issue #828 for mapper extraction reference

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.base_url = self.url = "https://www.bap1.uscourts.gov/bapopn"
        self.court_id = self.__module__

        # There are 29 historical pages as of development in Dec 2023
        # source indexes from 0
        self.back_scrape_iterable = range(29)[::-1]

    def _download(self, request_dict: Optional[Dict] = None) -> HtmlElement:
        """Gets the source's HTML

        For a normal periodic scraper, get the last page where
        most recent opinions are
        Backscraper will iterate over all available pages

        :param request_dict: unused in this scraper
        :return: HTML object from the downloaded page
        """
        if self.base_url == self.url:
            self.html = super()._download()
            self.url = self.html.xpath("//li/a/@href")[-1]

        return super()._download()

    def _download_backwards(self, page_number: int) -> None:
        """Method used by backscraper to download historical records

        :param page_number: an element of self.back_scrape_iterable
        :return: None
        """
        self.url = f"{self.base_url}?page={page_number}"

    def _process_html(self) -> None:
        """Parses a page's HTML into opinion dictionaries

        Most recent opinions are on the last rows of the table

        :return: None
        """
        for row in self.html.xpath("//tr[td]")[::-1]:
            docket_string = self.get_text_by_xpath(row, "td[1]")

            try:
                status = self.get_status_from_docket_string(docket_string)
            except SkipRowError:
                continue

            partial_docket_number = self.get_text_by_xpath(row, "td[2]")
            date_placeholder = self.get_text_by_xpath(row, "td[3]")
            lower_court = self.get_text_by_xpath(row, "td[4]/span")
            name = row.xpath("td[4]")[0].text.strip()

            docket = self.build_full_docket_number(
                partial_docket_number, lower_court
            )

            self.cases.append(
                {
                    "status": status,
                    "url": row.xpath("td[1]/a/@href")[0],
                    "docket": docket,
                    "date": date_placeholder,
                    "name": name,
                    "lower_court": lower_court,
                }
            )

    def build_full_docket_number(
        self, partial_docket_number: str, lower_court: str
    ) -> str:
        """Completes docket number with lower court abbreviation

        For each unique lower court in the opinions available on the source,
        a linked opinion PDF was opened and the abbrevation extracted

        :param partial_docket_number: The partial docket string
        :param lower_court: The lower court abbreviation
        :return: The full docket number
        """
        lower_court_abbreviation = self.lower_court_to_abbreviation.get(
            lower_court, ""
        )

        return f"BAP No. {lower_court_abbreviation} {partial_docket_number}"

    @staticmethod
    def get_status_from_docket_string(docket_string: str) -> str:
        """Extracts status implicit in partial docket number

        Examples of opinion names: 02-084P1.01A, 00-094P1, 21-021P

        :param docket_string
        :raises SkipRowError: when the docket refers to an order
        :return: status in `[Published, Unpublished, Unknown]`
        """
        if "P" in docket_string:
            status = "Published"
        elif "U" in docket_string:
            status = "Unpublished"
        elif "O" in docket_string:
            raise SkipRowError(
                f"Skipping row {docket_string} because it is an order"
            )
        else:
            status = "Unknown"

        return status

    @staticmethod
    def get_text_by_xpath(html_element: HtmlElement, xpath: str) -> str:
        """Extracts first text content from html element located by xpath

        :param html_element: anchor element for xpath selection
        :param xpath: xpath string
        :return: stripped text content
        """
        return html_element.xpath(xpath)[0].text_content().strip()

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Extract Date Filed from text

        :param scraped_text: Text of scraped content
        :return: Dict in the format expected by courtlistener,
                 containing date_filed
        """
        months = "|".join(calendar.month_name[1:])
        date_pattern = re.compile(rf"({months})\s+\d{{1,2}}\s?,?\s+\d{{4}}")
        match = re.search(date_pattern, scraped_text)
        date_extracted = match.group(0) if match else ""
        date_filed = re.sub(r"\s+", " ", date_extracted).strip()

        metadata = {
            "OpinionCluster": {
                "date_filed": convert_date_string(date_filed),
            },
        }
        return metadata



from datetime import date, datetime, timedelta

from dateutil.rrule import DAILY, rrule
from lxml.html import fromstring

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.base = "http://www.ca5.uscourts.gov"
        self.url = f"{self.base}/electronic-case-filing/case-information/current-opinions"
        self.court_id = self.__module__
        self.interval = 7
        self.data = {}
        self.headers = {
            "Host": "www.ca5.uscourts.gov",
            "Origin": self.base,
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7)",
            "Referer": self.url,
            "Connection": "keep-alive",
        }
        self.x = "ctl00$Body$C010$ctl00$ctl00"
        self.y = "ctl00_Body_C010_ctl00_ctl00"
        self.vs_xpath = "//input[@name='__VIEWSTATE']"
        self.ev_xpath = "//input[@name='__EVENTVALIDATION']"
        self.et_xpath = "//input[@name='__EVENTTARGET']"
        self.vsg_xpath = "//input[@name='__VIEWSTATEGENERATOR']"

        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=self.interval,  # Every interval days
                dtstart=date(2022, 7, 1),
                until=date(2022, 9, 1),
            )
        ]

    def _download(self):
        r = self.request["session"].get(self.url)
        html = fromstring(r.text)
        self._update_query_params(html)

        # Get first set of dates
        r = self.request["session"].post(self.url, data=self.data)
        return fromstring(r.text)

    def _process_html(self) -> None:
        """Process the html and extract out the opinions

        this is a larger function.  It processes the first page and then
        iterates over the remaining pages.  It requires an update to the params
        and there are plenty of janky names but its pretty fast and
        selenium free.

        :return: None
        """

        row_xpath = f"{self.y}_radGridOpinions_ctl00"
        more_rows = self.html.xpath(f"//tr[contains(@id, '{row_xpath}')]")
        for row in more_rows:
            self.cases.append(
                {
                    "date": row.xpath(".//td[3]")[0].text_content(),
                    "name": row.xpath(".//td[4]")[0].text_content(),
                    "url": row.xpath(".//td[2]/a")[0].attrib["href"],
                    "docket": row.xpath(".//td[2]/a")[0].attrib["title"],
                    "status": "Published"
                    if row.xpath(".//td[5]")[0].text_content() == "pub"
                    else "Unpublished",
                }
            )

        del self.data[f"{self.x}$btnSearch"]
        rad_script = f"{self.x}${self.x}$radGridOpinionsPanel|{self.x}$radGridOpinions$ctl00$ctl03$ctl01$ctl10"

        # switch to search mode for pagination
        self.data[f"{self.x}$searchMode"] = "search"
        self.data["__ASYNCPOST"] = "true"
        self.data["RadAJAXControlID"] = f"{self.y}_radAjaxManager1"
        self.data["ctl00$RadScriptManager1"] = rad_script

        last = self.html.xpath(
            "//div[@class='rgWrap rgNumPart']/a/span/text()"
        )[-1]

        page_content = None
        current_xp = "//a[@class='rgCurrentPage']/span/text()"
        while last != (current_page := self.html.xpath(current_xp)[0]):
            self._update_pagination_data(page_content, current_page)
            page_content = (
                self.request["session"]
                .post(self.url, headers=self.headers, data=self.data)
                .text
            )
            self.html = fromstring(page_content)
            rows = self.html.xpath(f"//tr[contains(@id, '{row_xpath}')]")
            for row in rows:
                self.cases.append(
                    {
                        "date": row.xpath(".//td[3]")[0].text_content(),
                        "name": row.xpath(".//td[4]")[0].text_content(),
                        "url": row.xpath(".//td[2]/a")[0].attrib["href"],
                        "docket": row.xpath(".//td[2]/a")[0].attrib["title"],
                        "status": "Published"
                        if row.xpath(".//td[5]")[0].text_content() == "pub"
                        else "Unpublished",
                    }
                )

    def _set_dates(self, case_date):
        """Set the backscrape dates"""
        d1 = case_date - timedelta(days=self.interval)
        e1 = case_date

        start_date_mdy = datetime.strftime(d1, "%m/%d/%Y")
        end_date_mdy = datetime.strftime(case_date, "%m/%d/%Y")
        start_date = str(
            {
                "valueAsString": f"{d1}-00-00-00",
                "lastSetTextBoxValue": f"{start_date_mdy}",
            }
        )
        end_date = str(
            {
                "valueAsString": f"{e1}-00-00-00",
                "lastSetTextBoxValue": f"{end_date_mdy}",
            }
        )
        self.data[f"{self.x}$startDate$dateInput"] = start_date_mdy
        self.data[f"{self.x}$endDate$dateInput"] = end_date_mdy
        self.data[f"{self.y}_startDate_dateInput_ClientState"] = start_date
        self.data[f"{self.y}_endDate_dateInput_ClientState"] = end_date
        self.data[f"{self.x}$btnSearch"] = "Search"
        self.data[
            f"{self.x}$radGridOpinions$ctl00$ctl03$ctl01$PageSizeComboBox"
        ] = "20"

    def _update_query_params(self, html):
        """Update the query parameters for next page"""
        self.data["__VIEWSTATE"] = html.xpath(self.vs_xpath)[0].attrib["value"]
        self.data["__EVENTVALIDATION"] = html.xpath(self.ev_xpath)[0].attrib[
            "value"
        ]
        self.data["__EVENTTARGET"] = html.xpath(self.et_xpath)[0].attrib[
            "value"
        ]
        self.data["__VIEWSTATEGENERATOR"] = html.xpath(self.vsg_xpath)[
            0
        ].attrib["value"]

    def _update_pagination_data(self, page_content, current):
        """Update subsequent pagination data if required."""
        target = self.html.xpath("//input[@class='rgPageNext']")[0].attrib[
            "name"
        ]
        if int(current) > 1:
            # After the first page - find view state etc here.
            viewstate = page_content.split("__VIEWSTATE|")[1].split("|")[0]
            valiation = page_content.split("__EVENTVALIDATION|")[1].split("|")[
                0
            ]
        else:
            viewstate = self.html.xpath(self.vs_xpath)[0].attrib["value"]
            valiation = self.html.xpath(self.ev_xpath)[0].attrib["value"]

        self.data["__EVENTTARGET"] = target
        self.data["__VIEWSTATE"] = viewstate
        self.data["__EVENTVALIDATION"] = valiation

    def _download_backwards(self, case_date):
        logger.info(
            "Running backscraper with date range: %s to %s"
            % (
                case_date - timedelta(days=self.interval),
                case_date,
            )
        )
        self._set_dates(case_date)



# Scraper for Texas Supreme Court
# CourtID: tex
# Court Short Name: TX
# Court Contacts:
#  - http://www.txcourts.gov/contact-us/
#  - Blake Hawthorne <Blake.Hawthorne@txcourts.gov>
#  - Eddie Murillo <Eddie.Murillo@txcourts.gov>
#  - Judicial Info <JudInfo@txcourts.gov>
# Author: Andrei Chelaru
# Reviewer: mlr
# History:
#  - 2014-07-10: Created by Andrei Chelaru
#  - 2014-11-07: Updated by mlr to account for new website.
#  - 2014-12-09: Updated by mlr to make the date range wider and more thorough.
#  - 2015-08-19: Updated by Andrei Chelaru to add backwards scraping support.
#  - 2015-08-27: Updated by Andrei Chelaru to add explicit waits
#  - 2021-12-28: Updated by flooie to remove selenium.

from datetime import date, timedelta
from typing import Optional

from juriscraper.AbstractSite import logger
from juriscraper.DeferringList import DeferringList
from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.checkbox = 0
        self.status = "Published"
        self.url = "https://search.txcourts.gov/CaseSearch.aspx?coa=cossup"

    def _set_parameters(
        self,
        view_state: str,
    ) -> None:
        """Set ASPX post parameters

        This method - chooses the court and date parameters.
        ctl00$ContentPlaceHolder1$chkListCourts$[KEY] is what selects a ct

         0: Supreme
         1: Court of Criminal Appeals
         2: 1st District Court of Appeals
         3: 2nd District Court of Appeals
         ...etc

        :param view_state: view state of the page
        :return: None
        """
        today = date.today()
        last_month = today - timedelta(days=30)
        last_month_str = last_month.strftime("%m/%d/%Y")
        today_str = today.strftime("%m/%d/%Y")

        date_param = (
            '{"enabled":true,"emptyMessage":"",'
            f'"validationText":"{last_month}-00-00-00",'
            f'"valueAsString":"{last_month}-00-00-00",'
            '"minDateStr":"1900-01-01-00-00-00",'
            '"maxDateStr":"2099-12-31-00-00-00",'
            f'"lastSetTextBoxValue":"{last_month_str}"'
            "}"
        )

        # The parameters required to filter in Texas
        self.parameters = {
            "ctl00$ContentPlaceHolder1$SearchType": "rbSearchByDocument",
            "ctl00$ContentPlaceHolder1$dtDocumentFrom": str(last_month),
            "ctl00$ContentPlaceHolder1$dtDocumentFrom$dateInput": last_month_str,
            "ctl00_ContentPlaceHolder1_dtDocumentFrom_dateInput_ClientState": date_param,
            "ctl00$ContentPlaceHolder1$dtDocumentTo": str(today),
            "ctl00$ContentPlaceHolder1$dtDocumentTo$dateInput": today_str,
            "ctl00$ContentPlaceHolder1$chkListDocTypes$0": "on",
            "ctl00$ContentPlaceHolder1$btnSearchText": "Search",
            "__VIEWSTATE": view_state,
            f"ctl00$ContentPlaceHolder1$chkListCourts${self.checkbox}": "on",
        }

    def _process_html(self) -> None:
        if not self.test_mode_enabled():
            # Make our post request to get our data
            self.method = "POST"
            view_state = self.html.xpath("//input[@id='__VIEWSTATE']")[0].get(
                "value"
            )
            self._set_parameters(view_state)
            self.html = super()._download()

        for row in self.html.xpath("//table[@class='rgMasterTable']/tbody/tr"):
            # In texas we also have to ping the case page to get the name
            # this is unfortunately part of the process.
            self.cases.append(
                {
                    "date": row.xpath(f".//td[2]")[0].text_content(),
                    "docket": row.xpath(f".//td[5]")[0].text_content().strip(),
                    "url": row.xpath(".//a")[1].get("href"),
                }
            )

    def _get_case_names(self) -> DeferringList:
        """Get case names using a deferring list."""
        seeds = []
        for row in self.html.xpath("//table[@class='rgMasterTable']/tbody/tr"):
            # In texas we also have to ping the case page to get the name
            # this is unfortunately part of the process.
            seeds.append(row.xpath(".//a")[2].get("href"))

        def get_name(link: str) -> Optional[str]:
            """Abstract out the case name from the case page."""
            if self.test_mode_enabled():
                return "No case names fetched during tests."
            html = self._get_html_tree_by_url(link)
            try:
                plaintiff = html.xpath(
                    '//label[contains(text(), "Style:")]/parent::div/following-sibling::div/text()'
                )[0].strip()
                defendant = html.xpath(
                    '//label[contains(text(), "v.:")]/parent::div/following-sibling::div/text()'
                )[0].strip()

                # In many cases the court leaves off the plaintiff (The State of Texas).  But
                # in some of these cases the appellant is ex parte.  So we need to
                # add the state of texas in some cases but not others.
                if defendant:
                    return titlecase(f"{plaintiff} v. {defendant}")
                elif "WR-" not in link:
                    return titlecase(f"{plaintiff} v. The State of Texas")
                else:
                    return titlecase(f"{plaintiff}")
            except IndexError:
                logger.warning(f"No title or defendant found for {self.url}")
                return None

        return DeferringList(seed=seeds, fetcher=get_name)



from datetime import datetime

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = datetime.today().year
        self.url = f"https://www.courts.wa.gov/opinions/index.cfm?fa=opinions.byYear&fileYear={self.year}&crtLevel=S&pubStatus=PUB"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath('//tr/td[@valign="top"]/..')[::-1]:
            if len(row.xpath(".//td")) != 4:
                continue
            date, middle, name, op_type = row.xpath(".//td")
            self.cases.append(
                {
                    "date": date.text_content(),
                    "url": middle.xpath("a")[1].get("href"),
                    "name": name.text_content().replace("* ", ""),
                    "docket": middle.xpath("a")[0].text_content(),
                }
            )



"""Scraper for Nevada Supreme Court
CourtID: nev

History:
    - 2023-12-13: Updated by William E. Palin
"""


from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://publicaccess.nvsupremecourt.us/WebSupplementalAPI/api/AdvanceOpinions"
        self.status = "Published"

    def _download(self):
        if self.test_mode_enabled():
            import json

            return json.load(open(self.url))

        return (
            self.request["session"]
            .get(
                self.url,
                headers=self.request["headers"],
                verify=self.request["verify"],
                timeout=60,
            )
            .json()
        )

    def _process_html(self):
        for case in self.html:
            if "COA" in case["caseNumber"]:
                continue
            self.cases.append(
                {
                    "name": case["caseTitle"],
                    "docket": case["caseNumber"],
                    "date": case["date"],
                    "url": case["docurl"],
                }
            )



from juriscraper.opinions.united_states.state import washctapp_p


class Site(washctapp_p.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = f"https://www.courts.wa.gov/opinions/index.cfm?fa=opinions.byYear&fileYear={self.year}&crtLevel=C&pubStatus=UNP"
        self.status = "Unpublished"



# Scraper for Oklahoma Attorney General Opinions
# CourtID: oklaag
# Court Short Name: OK
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-05

from datetime import date

from lxml import html

from juriscraper.opinions.united_states.state import okla

## WARNING: THIS SCRAPER IS FAILING:
## This scraper is succeeding in development, but
## is failing in production.  We are not exactly
## sure why, and suspect that the hosting court
## site may be blocking our production IP and/or
## throttling/manipulating requests from production.


class Site(okla.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        year = date.today().year
        self.url = f"http://www.oscn.net/applications/oscn/Index.asp?ftdb=STOKAG&year={year}&level=1"
        self.court_id = self.__module__
        self.status = "Unpublished"

    def _process_html(self):
        for row in self.html.xpath("//div/p['@class=document']")[::-1]:
            if "OK" not in row.text_content():
                continue
            if "EMAIL" in row.text_content():
                continue
            citation, date, name = row.text_content().split(",", 2)
            self.cases.append(
                {
                    "date": date,
                    "name": name,
                    "docket": "",
                    "url": row.xpath(".//a")[0].get("href"),
                    "citation": citation,
                }
            )

    @staticmethod
    def cleanup_content(content):
        tree = html.fromstring(content)
        core_element = tree.xpath('//div[@id="oscn-content"]/div')[0]
        return html.tostring(
            core_element, pretty_print=True, encoding="unicode"
        ).encode("utf-8")



import re
from urllib.parse import quote

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.utcourts.gov/opinions/appopin/index.htm"
        self.court_id = self.__module__
        self.regex = r"Case No. (.*?), Filed (.*?), (\d{4} UT App \d+)"

    def _process_html(self) -> None:
        for row in self.html.xpath("//a[@class='pdf']/parent::p"):
            link = row.xpath("./a")[0]
            x = " ".join(row.xpath(".//text()")).strip()
            if "Superseded" in x:
                continue
            m = re.search(self.regex, x)
            if not m:
                continue
            date = m.groups()[1]
            if "Filed" in date:
                date = date.replace("Filed", "").strip()
            citation = m.groups()[2]
            docket_number = m.groups()[0]
            self.cases.append(
                {
                    "date": date,
                    "name": row.xpath(".//text()")[0],
                    "citation": citation,
                    "url": quote(link.attrib["href"], safe=":/"),
                    "docket": docket_number,
                    "status": "Published",
                }
            )



"""Scraper for the Supreme Court of Ohio
CourtID: ohio
Court Short Name: Ohio
Author: Andrei Chelaru
Reviewer: mlr
History:
 - Stubbed out by Brian Carver
 - 2014-07-30: Finished by Andrei Chelaru
 - 2015-07-31: Redone by mlr to use ghost driver. Alas, their site used to be
               great, but now it's terribly frustrating.
 - 2021-12-28: Remove selenium by flooie
"""
from datetime import date

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_index = 0
        self.year = date.today().year
        self.url = "https://www.supremecourtofohio.gov/rod/docs/"
        self.court_id = self.__module__

    def _set_parameters(
        self,
        event_validation: str,
        view_state: str,
    ) -> None:
        """Set the parameters for the search

        :param: event_validation: the event validation token
        :param: view_state: the view state token
        :return: None
        """
        self.parameters = {
            "__VIEWSTATEENCRYPTED": "",
            "ctl00$MainContent$ddlCourt": f"{self.court_index}",
            "ctl00$MainContent$ddlDecidedYearMin": f"{self.year}",
            "ctl00$MainContent$ddlDecidedYearMax": f"{self.year}",
            "ctl00$MainContent$ddlCounty": "0",
            "ctl00$MainContent$btnSubmit": "Submit",
            "ctl00$MainContent$ddlRowsPerPage": "50",
            "__EVENTVALIDATION": event_validation,
            "__VIEWSTATE": view_state,
        }
        self.url = "https://www.supremecourt.ohio.gov/rod/docs/"
        self.method = "POST"

    def _process_html(self) -> None:
        """Process the HTML and extract the data"""
        if not self.test_mode_enabled():
            event_validation = self.html.xpath(
                "//input[@id='__EVENTVALIDATION']"
            )[0].get("value")
            view_state = self.html.xpath("//input[@id='__VIEWSTATE']")[0].get(
                "value"
            )
            self._set_parameters(
                event_validation,
                view_state,
            )
            self.html = self._download()

        # Skip the header rows and the footer rows
        for row in self.html.xpath(
            ".//table[@id='MainContent_gvResults']//tr"
        )[3:-2]:
            # Filter out the case announcements and rulings (ie non-opinions)
            if not row.xpath(".//td[2]//text()")[0].strip():
                continue
            self.cases.append(
                {
                    "judge": row.xpath(".//td[4]//text()")[0],
                    "docket": row.xpath(".//td[2]//text()")[0],
                    "date": row.xpath(".//td[6]//text()")[0],
                    "name": row.xpath(".//a/text()")[0],
                    "url": row.xpath(".//a")[0].get("href"),
                    "citation": row.xpath(".//td[8]//text()")[0],
                    "summary": row.xpath(".//td[3]//text()")[0],
                    "status": "Published",
                }
            )



# Scraper for Superior Court of the Virgin Islands (unpublished)
# CourtID: visuper_u

from juriscraper.opinions.united_states.territories import visuper_p


class Site(visuper_p.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "https://superior.vicourts.org/court_opinions/unpublished_opinions"
        )
        self.status = "Unpublished"



import json
import re
from datetime import date, timedelta
from typing import Any, Dict

from dateutil.rrule import DAILY, rrule

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.govinfo.gov/wssearch/search"
        self.court_id = self.__module__
        self.td = date.today()
        today = self.td.strftime("%Y-%m-%d")
        last_month = (self.td - timedelta(days=31)).strftime("%Y-%m-%d")
        self.parameters = {
            "facets": {
                "accodenav": [
                    "USCOURTS",
                ],
                "governmentauthornav": [
                    "United States Court of Appeals for the Fourth Circuit",
                ],
            },
            "filterOrder": [
                "accodenav",
                "governmentauthornav",
            ],
            "facetToExpand": "governmentauthornav",
            "offset": 0,
            "pageSize": "100",
            "sortBy": "2",
            "query": f"publishdate:range({last_month},{today})",
            "browseByDate": True,
            "historical": False,
        }
        self.method = "POST"
        self.json = {}

        self.interval = 14
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=self.interval,  # Every interval days
                dtstart=date(2022, 1, 1),
                until=date(2022, 7, 1),
            )
        ]

    def _download(self, request_dict={}):
        if self.test_mode_enabled():
            with open(self.url) as file:
                self.json = json.load(file)
        else:
            self.json = (
                self.request["session"]
                .post(
                    self.url,
                    json=self.parameters,
                )
                .json()
            )

    def _process_html(self) -> None:
        """Process CA4 Opinions

        :return: None
        """
        for row in self.json["resultSet"]:
            package_id = row["fieldMap"]["packageid"]
            docket = row["line1"].split()[0]
            date_str = row["line2"].split("day, ")[1].strip(".")
            url = f"https://www.govinfo.gov/content/pkg/{package_id}/pdf/{package_id}-0.pdf"

            self.cases.append(
                {
                    "docket": docket,
                    "name": row["fieldMap"]["title"],
                    "url": url,
                    "date": date_str,
                    "status": "Unknown",
                }
            )

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Pass scraped text into function and return precedential status

        :param scraped_text: Text of scraped content
        :return: metadata
        """
        if re.findall(r"\bPUBLISHED\b", scraped_text):
            status = "Published"
        elif re.findall(r"\bUNPUBLISHED\b", scraped_text):
            status = "Unpublished"
        else:
            status = "Unknown"
        metadata = {
            "OpinionCluster": {
                "precedential_status": status,
            },
        }
        return metadata

    def _download_backwards(self, dt) -> None:
        """Download backward over time method

        :param dt: Datetime object
        :return: None
        """
        start = (dt - timedelta(days=7)).strftime("%Y-%m-%d")
        end = dt.strftime("%Y-%m-%d")
        self.parameters["query"] = f"publishdate:range({start},{end})"
        self.html = self._download()



"""
Scraper template for the 'Other Courts' of the NY Reporter
Court Contact: phone: (518) 453-6900
Author: Gianfranco Rossi
History:
 - 2024-01-05, grossir: created
"""
import re
from datetime import date
from itertools import chain
from typing import Any, Dict, List, Optional

from dateutil.rrule import MONTHLY, rrule
from lxml.html import fromstring

from juriscraper.AbstractSite import logger
from juriscraper.lib.judge_parsers import normalize_judge_string
from juriscraper.lib.string_utils import harmonize
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    court_regex: str  # to be defined on inheriting classes
    base_url = "https://nycourts.gov/reporter/slipidx/miscolo.shtml"
    first_opinion_date = date(2003, 12, 1)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.court_id = self.__module__
        self.url = self.build_url()

        date_keys = rrule(
            MONTHLY, dtstart=self.first_opinion_date, until=date(2023, 12, 30)
        )
        self.back_scrape_iterable = [i.date() for i in date_keys]

    def build_url(self, target_date: Optional[date] = None) -> str:
        """URL as is loads most recent month page
        There is an URL for each month of each year back to Dec 2003

        :param target_date: used to extract month and year for backscraping
        :returns str: formatted url
        """
        if not target_date:
            return self.base_url

        end = f"_{target_date.year}_{target_date.strftime('%B')}.shtml"

        return self.base_url.replace(".shtml", end)

    def is_court_of_interest(self, court: str) -> bool:
        """'Other Courts' of NY Reporter consists of 10 different
        family of sources. Each family has an scraper that inherits
        from this class and defines a `court_regex` to capture those
        that belong to its family

        For example
        "Civ Ct City NY, Queens County" and "Civ Ct City NY, NY County"
        belong to nycivct family

        :param court: court name
        :return: true if court name matches
                family of courts of calling scraper
        """
        return bool(re.search(self.court_regex, court))

    def _process_html(self) -> None:
        """Parses a page's HTML into opinion dictionaries

        :return: None
        """
        row_xpath = "//table[caption]//tr[position()>1 and td]"
        for row in self.html.xpath(row_xpath):
            court = re.sub(
                r"\s+", " ", row.xpath("td[2]")[0].text_content()
            ).strip(", ")

            if not self.is_court_of_interest(court):
                logger.debug("Skipping %s", court)
                continue

            url = row.xpath("td[1]/a/@href")[0]
            name = harmonize(row.xpath("td[1]/a")[0].text_content())
            opinion_date = row.xpath("td[3]")[0].text_content()
            slip_cite = row.xpath("td[4]")[0].text_content()
            status = "Unpublished" if "(U)" in slip_cite else "Published"

            self.cases.append(
                {
                    "name": name,
                    "date": opinion_date,
                    "status": status,
                    "url": url,
                    "citation": slip_cite,
                    "child_court": court,
                }
            )

    def _get_docket_numbers(self) -> List[str]:
        """Overriding from OpinionSiteLinear, since docket numbers are
        not in the HTML and they are required

        We will get them on the extract_from_text stage on courtlistener

        :return: list of empty strings values
        """
        return ["" for _ in self.cases]

    def _download_backwards(self, target_date: date) -> None:
        """Method used by backscraper to download historical records

        :param target_date: an element of self.back_scrape_iterable
        :return: None
        """
        self.url = self.build_url(target_date)

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Extract values from opinion's text

        :param scraped_text: pdf or html string contents
        :return: dict where keys match courtlistener model objects
        """
        pattern = r"Judge:\s?(.+)|([\w .,]+), [JS]\.\s"
        judge = self.match(scraped_text, pattern)

        pattern = r"</table><br><br\s?/?>\s?(.*)\r?\n|Docket Number:\s?(.+)"
        docket_number = self.match(scraped_text, pattern)

        pattern = r"\[(?P<volume>\d+) (?P<reporter>Misc 3d) (?P<page>.+)\]"
        cite_match = re.search(pattern, scraped_text[:2000])

        # Only for .htm links
        full_case = None
        if scraped_text.find("<table") != -1:
            # replace <br> with newlines because text_content() replaces <br>
            # with whitespace. If not, case names would lack proper separation
            scraped_text = scraped_text.replace("<br>", "\n")
            full_case = fromstring(scraped_text).xpath("//table")
            full_case = full_case[1].text_content() if full_case else ""

        metadata = {
            "Docket": {"docket_number": docket_number},
        }

        if judge:
            metadata["Opinion"] = {
                "author_str": normalize_judge_string(judge)[0]
            }
        if cite_match:
            metadata["Citation"] = cite_match.groupdict("")
            metadata["Citation"]["type"] = 2  # 'State' type in courtlistener
        if full_case:
            full_case = harmonize(full_case)
            metadata["Docket"]["case_name_full"] = full_case
            metadata["OpinionCluster"] = {"case_name_full": full_case}

        return metadata

    @staticmethod
    def match(scraped_text: str, pattern: str) -> str:
        """Returns first match

        :param scraped_text: HTML or PDF string content
        :param pattern: regex string

        :returns: first match
        """
        m = re.findall(pattern, scraped_text)
        r = list(filter(None, chain.from_iterable(m)))
        return r[0].strip() if r else ""



from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.njcourts.gov/attorneys/opinions/supreme"
        self.status = "Published"

    def _process_html(self) -> None:
        """Process the html and extract out the opinions

        :return: None
        """
        for row in self.html.xpath("//div[@class='card-body']"):
            url = row.xpath(".//a[@class='text-underline-hover']/@href")[0]
            name = row.xpath(".//a[@class='text-underline-hover']/text()")[0]
            if "njtax" in self.court_id:
                pass
            else:
                name, other = name.split("(", 1)
            docket = row.xpath(
                ".//div[@class='badge badge-default rounded-0 one-line-truncate me-1 mt-1']/text()"
            )[0]
            date = row.xpath(
                ".//div[@class='col-lg-12 small text-muted mt-2']/text()"
            )[0]
            self.cases.append(
                {
                    "date": date,
                    "docket": docket,
                    "name": titlecase(name.strip()),
                    "url": url,
                }
            )
